@@ -4,6 +4,7 @@
  *  pcb-rnd, interactive printed circuit board design
  *  (this file is based on PCB, interactive printed circuit board design)
  *  Copyright (C) 1994,1995,1996,2004,2006 Thomas Nau
+ *  Copyright (C) 2023 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -46,10 +47,12 @@
 #include <sys/types.h>
 #include <assert.h>
 #include <librnd/core/compat_fs.h>
+#include <librnd/core/compat_fs_dir.h>
 #include <librnd/core/compat_misc.h>
 #include <librnd/core/compat_lrealpath.h>
 #include <librnd/core/globalconst.h>
 #include <librnd/core/safe_fs.h>
+#include <librnd/core/safe_fs_dir.h>
 
 #include <genvector/gds_char.h>
 
@@ -361,6 +364,8 @@ int rnd_is_path_abs(const char *fn)
 #endif
 }
 
+#define ISSEP(c) (((c) == '/') || ((c) == RND_DIR_SEPARATOR_C))
+
 char *rnd_dirname(const char *path)
 {
 	long len = strlen(path);
@@ -372,7 +377,7 @@ char *rnd_dirname(const char *path)
 
 	memcpy(res, path, len+1);
 	for(s = res + len - 1; s >= res; s--) {
-		if ((*s == '/') || (*s == RND_DIR_SEPARATOR_C)) {
+		if (ISSEP(*s)) {
 			*s = '\0';
 			return res;
 		}
@@ -395,3 +400,141 @@ char *rnd_dirname_real(const char *path)
 	return res;
 }
 
+
+int rnd_mktempdir(rnd_design_t *dsg, gds_t *dst, const char *prefix)
+{
+	const char *tmpdir = getenv("TMPDIR");
+	long tmpdir_len, prefix_len;
+	char *end;
+	int tries;
+
+	TODO("what about win32? this is broken in rnd_tempfile_name_new() as well");
+	if (tmpdir == NULL) {
+		tmpdir = "/tmp";
+		tmpdir_len = 4;
+	}
+	else
+		tmpdir_len = strlen(tmpdir);
+
+	prefix_len = strlen(prefix);
+	gds_enlarge(dst, tmpdir_len + 1 + prefix_len + 32);
+	dst->used = 0;
+	if (dst->array == NULL)
+		return -1;
+
+	end = dst->array;
+	memcpy(end, tmpdir, tmpdir_len); end += tmpdir_len;
+	*end = RND_DIR_SEPARATOR_C; end++;
+	memcpy(end, prefix, prefix_len); end += prefix_len;
+	*end = '.'; end++;
+
+	for(tries = 32; tries > 0; tries--) {
+		int n;
+		char *s = end;
+		char pool[] = "abcdefghijklmnopqrstuvwxyz0123456789";
+		for(n = 0; n < 12; n++) {
+			*s = pool[rnd_rand() % (sizeof(pool)-1)];
+			s++;
+		}
+		*s = '\0';
+		if (rnd_mkdir(dsg, dst->array, 0750) == 0) {
+			dst->used = s - dst->array;
+			return 0;
+		}
+	}
+
+	gds_uninit(dst);
+	return -1;
+}
+
+int rnd_rmtree(rnd_design_t *dsg, gds_t *dst)
+{
+	if (rnd_is_dir(dsg, dst->array)) {
+		DIR *dir = rnd_opendir(dsg, dst->array);
+		struct dirent *de;
+
+		while((de = rnd_readdir(dir)) != NULL) {
+			int save = dst->used;
+
+			if (de->d_name[0] == '.') { /* skip . and .. */
+				if (de->d_name[1] == '\0') continue;
+				if ((de->d_name[1] == '.') && (de->d_name[2] == '\0')) continue;
+			}
+
+			gds_append_str(dst, de->d_name);
+			rnd_rmtree(dsg, dst);
+			dst->array[save] = '\0';
+			dst->used = save;
+		}
+		rnd_closedir(dir);
+	}
+	rnd_remove(dsg, dst->array);
+	return 0;
+}
+
+int rnd_rmtempdir(rnd_design_t *dsg, gds_t *dst)
+{
+	int res = rnd_rmtree(dsg, dst);
+	gds_uninit(dst);
+	return res;
+}
+
+char *rnd_relative_path_files(const char *pth, const char *relto)
+{
+	char *freeme1 = NULL, *freeme2 = NULL;
+	const char *s, *s1, *s2;
+	int offs, commsep = -1;
+	gds_t res = {0};
+	
+
+	/* make sure both inputs are full paths */
+	if (!rnd_is_path_abs(pth))
+		pth = freeme1 = rnd_lrealpath(pth);
+	if (!rnd_is_path_abs(relto))
+		relto = freeme2 = rnd_lrealpath(relto);
+
+	/* calculate common part */
+	for(offs = 0, s1 = pth, s2 = relto; *s1 == *s2; s1++, s2++, offs++)
+		if (ISSEP(*s1) && ISSEP(*s2))
+			commsep = offs;
+
+	assert(commsep >= 0); /* both paths are absolute, worst case first char match */
+
+	/* add all the ../ */
+	for(s = relto+commsep+1; *s != '\0'; s++) {
+		if (ISSEP(s[0]) && !ISSEP(s[1]))
+			gds_append_str(&res, "../");
+	}
+
+	/* append partial path to dest file */
+	gds_append_str(&res, pth+commsep+1);
+
+	free(freeme1);
+	free(freeme2);
+	return res.array;
+}
+
+const char *rnd_basename(const char *path)
+{
+	char *res = strrchr(path, '/');
+	if (res == NULL) return path;
+	return res+1;
+}
+
+const char *rnd_parent_dir_name(const char *fullpath)
+{
+	const char *bn;
+	int seps = 0;
+
+	for(bn = fullpath + strlen(fullpath) - 1; bn > fullpath; bn--) {
+		if (ISSEP(bn[0]) && !ISSEP(bn[1])) {
+			seps++;
+			if (seps == 2) {
+				bn++;
+				return bn;
+			}
+		}
+	}
+
+	return bn;
+}
