@@ -2,7 +2,7 @@
  *                            COPYRIGHT
  *
  *  pcb-rnd, interactive printed circuit board design
- *  Copyright (C) 2016,2019..2021 Tibor 'Igor2' Palinkas
+ *  Copyright (C) 2016,2019..2023 Tibor 'Igor2' Palinkas
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -37,6 +37,7 @@
 #include <librnd/core/color.h>
 #include <librnd/core/conf.h>
 #include <librnd/core/conf_hid.h>
+#include <librnd/core/conf_multi.h>
 #include <librnd/core/hid_cfg.h>
 #include <librnd/core/misc_util.h>
 #include <librnd/core/error.h>
@@ -49,6 +50,7 @@
 #include <librnd/core/fptr_cast.h>
 #include <librnd/core/safe_fs_dir.h>
 #include <librnd/core/event.h>
+#include <librnd/core/rnd_conf.h>
 
 static const char *flcat = "conf";
 
@@ -58,7 +60,7 @@ static const char conf_cookie_al[] = "core/conf/anyload";
 static htsi_t conf_interns;
 static int conf_files_inited = 0;
 
-extern void rnd_pcbhl_conf_postproc(void);
+extern void rnd_conf_postproc(void);
 
 int rnd_conf_in_production = 0;
 
@@ -93,6 +95,12 @@ long rnd_conf_main_root_replace_cnt[RND_CFR_max_alloc]; /* number of times the r
 int rnd_conf_main_root_lock[RND_CFR_max_alloc];
 int rnd_conf_lht_dirty[RND_CFR_max_alloc];
 
+/* number of edits done on this subtree; multi: this central value is not
+   saved per design, but each design has its own. For shared docs if the
+   design's own edits is lower than this value, a merge is needed */
+long rnd_conf_lht_edits[RND_CFR_max_alloc];
+int rnd_conf_edits_lock = 0;
+
 /* Plugin config: only plugin configuration is accepted; never edited, only
    merged in. Merge takes two steps: first all files per role are merged into
    a single rnd_conf_plug_root[R] (lihata level merge), then rnd_conf_plug_root[R]
@@ -101,7 +109,9 @@ int rnd_conf_lht_dirty[RND_CFR_max_alloc];
 lht_doc_t *rnd_conf_plug_root[RND_CFR_max_alloc];
 
 
-htsp_t *rnd_conf_fields = NULL;
+htsp_t *rnd_conf_fields = NULL;         /* current design's config */
+htsp_t *rnd_conf_fields_master = NULL;  /* master config: new design's own config is copied from here*/
+
 const int rnd_conf_default_prio[] = {
 /*	RND_CFR_INTERNAL */   100,
 /*	RND_CFR_SYSTEM */     200,
@@ -648,7 +658,7 @@ int rnd_conf_merge_patch_text(rnd_conf_native_t *dest, lht_node_t *src, int prio
 		dest->prop[0].prio = prio;
 		dest->prop[0].src  = src;
 		dest->used         = 1;
-		dest->rnd_conf_rev     = rnd_conf_rev;
+		dest->rnd_conf_rev = rnd_conf_rev;
 	}
 	return 0;
 }
@@ -830,38 +840,59 @@ int rnd_conf_merge_patch_list(rnd_conf_native_t *dest, lht_node_t *src_lst, int
 
 int rnd_conf_merge_patch_recurse(lht_node_t *sect, rnd_conf_role_t role, int default_prio, rnd_conf_policy_t default_policy, const char *path_prefix);
 
-static void conf_warn_unknown_paths(const char *path, lht_node_t *n)
+static int conf_warn_unknown_paths_(const char *path, lht_node_t *n, rnd_conf_ignore_t *tbl)
 {
 	rnd_conf_ignore_t *i;
 
+	for(i = tbl; i->name != NULL; i++) {
+		if (strncmp(path, i->name, i->len) == 0) {
+			if (i->warned)
+				return 1; /* do not warn again */
+			i->warned = 1;
+			if (i->msg != NULL) { /* throw own error */
+				rnd_hid_cfg_error(n, i->msg);
+				return 1;
+			}
+			return 0; /* first warning: throw original error */
+		}
+	}
+
+	return 0; /* not found in the table throw original error */
+}
+
+static rnd_conf_ignore_t rnd_own_conf_ignores[] = {
+	{"plugins/hid_gtk/local_grid", 26, 0, "config node plugins/hid_gtk/local_grid got renamed to editor/local_grid, please edit your config file\n"},
+	{"plugins/hid_gtk/global_grid", 27, 0, "config node plugins/hid_gtk/global_grid got renamed to editor/global_grid, please edit your config file\n"},
+	{NULL, 0, 0, NULL}
+};
+
+static void conf_warn_unknown_paths(const char *path, lht_node_t *n)
+{
+	if (conf_warn_unknown_paths_(path, n, rnd_own_conf_ignores) != 0)
+		return;
+
 	if ((strncmp(path, "plugins/", 8) == 0) || (strncmp(path, "utils/", 6) == 0))
 		return; /* it is normal to have configuration for plugins and utils not loaded - ignore these */
 
 	if (rnd_app.conf_ignores != NULL) {
-		for(i = rnd_app.conf_ignores; i->name != NULL; i++) {
-			if (strncmp(path, i->name, i->len) == 0) {
-				if (i->warned)
-					return; /* do not warn again */
-				i->warned = 1;
-				break;
-			}
-		}
+		if (conf_warn_unknown_paths_(path, n, rnd_app.conf_ignores) != 0)
+			return;
 	}
 
 	rnd_hid_cfg_error(n, "conf error: lht->bin conversion: can't find path '%s'\n(it may be an obsolete setting, check your lht)\n", path);
 }
 
 /* returns 1 if the config node should be ignored */
-static int conf_board_ignore(const char *path, lht_node_t *n)
+static int conf_prj_dsg_ignore(const char *path, lht_node_t *n)
 {
 	rnd_conf_ignore_t *i;
 
-	if (rnd_app.conf_board_ignores != NULL) {
-		for(i = rnd_app.conf_board_ignores; i->name != NULL; i++) {
+	if (rnd_app.conf_prj_dsg_ignores != NULL) {
+		for(i = rnd_app.conf_prj_dsg_ignores; i->name != NULL; i++) {
 			if (strncmp(path, i->name, i->len) == 0) {
 				if (!i->warned) {
 					i->warned = 1;
-					rnd_hid_cfg_error(n, "conf error: lht->bin conversion: path '%s' from board or project file\nis ignored (probably due to security considerations)\n", path);
+					rnd_hid_cfg_error(n, "conf error: lht->bin conversion: path '%s' from design or project file\nis ignored (probably due to security considerations)\n", path);
 				}
 				return 1;
 			}
@@ -882,7 +913,7 @@ int rnd_conf_merge_patch_item(const char *path, lht_node_t *n, rnd_conf_role_t r
 	}
 
 	if ((role == RND_CFR_DESIGN) || (role == RND_CFR_PROJECT)) {
-		if (conf_board_ignore(path, n))
+		if (conf_prj_dsg_ignore(path, n))
 			return 0;
 	}
 
@@ -1092,6 +1123,19 @@ static void conf_field_clear(rnd_conf_native_t *f)
 	f->rnd_conf_rev = rnd_conf_rev;
 }
 
+static char *get_homedir(void)
+{
+	char *homedir = getenv("HOME");
+	if (homedir == NULL)
+		homedir = getenv("USERPROFILE");
+	return homedir;
+}
+
+void rnd_conf_postproc(void)
+{
+	rnd_conf_force_set_str(rnd_conf.rc.path.home, get_homedir()); rnd_conf_ro("rc/path/home");
+}
+
 int rnd_conf_rev = 0;
 void rnd_conf_update(const char *path, int arr_idx)
 {
@@ -1136,7 +1180,7 @@ void rnd_conf_update(const char *path, int arr_idx)
 	rnd_conf_merge_all(path);
 	if (rnd_app.conf_core_postproc != NULL)
 		rnd_app.conf_core_postproc();
-	rnd_pcbhl_conf_postproc();
+	rnd_conf_postproc();
 
 	if (path == NULL) {
 		htsp_entry_t *e;
@@ -1228,28 +1272,28 @@ lht_node_t *rnd_conf_lht_get_first_pol(rnd_conf_role_t target, rnd_conf_policy_t
 	return conf_lht_get_first_(rnd_conf_main_root[target]->root, pol, create, RND_POL_OVERWRITE);
 }
 
-lht_node_t *rnd_pref_ensure_conf_root(rnd_hidlib_t *hidlib, rnd_conf_role_t role)
+lht_node_t *rnd_pref_ensure_conf_root(rnd_design_t *hidlib, rnd_conf_role_t role)
 {
 	lht_node_t *m;
 
 	m = rnd_conf_lht_get_first(role, 0);
 	if (m == NULL) {
 		if (role == RND_CFR_PROJECT) {
-			const char *pcb_fn = (hidlib == NULL ? NULL : hidlib->filename);
-			const char *try, *fn = rnd_conf_get_project_conf_name(NULL, pcb_fn, &try);
+			const char *design_fn = (hidlib == NULL ? NULL : hidlib->loadname);
+			const char *try, *fn = rnd_conf_get_project_conf_name(NULL, design_fn, &try);
 			if (fn == NULL) {
-				rnd_message(RND_MSG_ERROR, "Failed to create the project file\n");
+				rnd_message(RND_MSG_ERROR, "Failed to create the project file: can't determine project file name\nIs your design saved so it has a parent directory?\n");
 				return NULL;
 			}
 			rnd_conf_reset(role, fn);
 			rnd_conf_makedirty(role);
-			rnd_conf_save_file(hidlib, fn, pcb_fn, role, NULL);
+			rnd_conf_save_file(hidlib, fn, design_fn, role, NULL);
 			m = rnd_conf_lht_get_first(role, 0);
 			if (m == NULL) {
-				rnd_message(RND_MSG_ERROR, "Failed to create the project file %s\n", fn);
+				rnd_message(RND_MSG_ERROR, "Failed to create the project file '%s'\n", fn);
 				return NULL;
 			}
-				rnd_message(RND_MSG_INFO, "Created the project file\n");
+			rnd_message(RND_MSG_INFO, "Created the project file '%s'\n", fn);
 		}
 		else {
 			rnd_message(RND_MSG_ERROR, "Failed to create config file for role %s\n", rnd_conf_role_name(role));
@@ -1371,9 +1415,19 @@ void rnd_conf_load_project(const char *project_fn, const char *design_fn)
 }
 
 
-static rnd_conf_native_t *rnd_conf_alloc_field_(void *value, int array_size, rnd_conf_native_type_t type, const char *path, const char *desc, rnd_conf_flag_t flags)
+static rnd_conf_native_t *rnd_conf_alloc_field_(void *value, int array_size, rnd_conf_native_type_t type, const char *path, const char *desc, rnd_conf_flag_t flags, int alloc4shared)
 {
-	rnd_conf_native_t *node = calloc(sizeof(rnd_conf_native_t), 1);
+	rnd_conf_native_t *node;
+
+	if (alloc4shared) {
+		/* overallocate to store shared locally - useful when storing for master */
+		node = calloc(sizeof(rnd_conf_native_t) + sizeof(rnd_conf_nat_shared_t), 1);
+		node->shared = (rnd_conf_nat_shared_t *)&node->shared_static;
+		vtp0_init(&(node->shared->hid_data));
+		vtp0_init(&(node->shared->hid_callbacks));
+	}
+	else
+		node = calloc(sizeof(rnd_conf_native_t), 1);
 
 	node->array_size  = array_size;
 	node->type        = type;
@@ -1382,8 +1436,6 @@ static rnd_conf_native_t *rnd_conf_alloc_field_(void *value, int array_size, rnd
 	node->description = desc;
 	node->hash_path   = path;
 	node->flags       = flags;
-	vtp0_init(&(node->hid_data));
-	vtp0_init(&(node->hid_callbacks));
 
 	return node;
 }
@@ -1392,18 +1444,20 @@ rnd_conf_native_t *rnd_conf_reg_field_(void *value, int array_size, rnd_conf_nat
 {
 	rnd_conf_native_t *node;
 
-	if (rnd_conf_fields == NULL) {
-		rnd_conf_fields = htsp_alloc(strhash, strkeyeq);
-		assert(rnd_conf_fields != NULL);
-	}
+	if (rnd_conf_fields_master == NULL)
+		rnd_conf_fields = rnd_conf_fields_master = htsp_alloc(strhash, strkeyeq);
+
+	assert(rnd_conf_fields != NULL);
 	assert(array_size >= 1);
 
 	assert(htsp_get(rnd_conf_fields, path) == NULL);
-	node = rnd_conf_alloc_field_(value, array_size, type, path, desc, flags);
+	node = rnd_conf_alloc_field_(value, array_size, type, path, desc, flags, 1);
 
-	htsp_set(rnd_conf_fields, (char *)path, node);
+	htsp_set(rnd_conf_fields_master, (char *)path, node);
 	rnd_conf_hid_global_cb(node, -1, new_item_post);
 
+	rnd_conf_multi_copy_master_to_per_dsg(node);
+
 	return node;
 }
 
@@ -1446,12 +1500,18 @@ void rnd_conf_free_native(rnd_conf_native_t *node)
 	if (node->random_flags.dyn_val)
 		free(node->val.any);
 
-	vtp0_uninit(&(node->hid_data));
-	vtp0_uninit(&(node->hid_callbacks));
+	if (node->shared != NULL) {
+		vtp0_uninit(&(node->shared->hid_data));
+		vtp0_uninit(&(node->shared->hid_callbacks));
+	}
 	free(node->prop);
 	free(node);
 }
 
+
+#define rnd_conf_fields_foreach_master(e) \
+	for (e = htsp_first(rnd_conf_fields_master); e; e = htsp_next(rnd_conf_fields_master, e))
+
 void rnd_conf_unreg_fields(const char *prefix)
 {
 	int len = strlen(prefix);
@@ -1459,18 +1519,33 @@ void rnd_conf_unreg_fields(const char *prefix)
 
 	assert(prefix[len-1] == '/');
 
-	rnd_conf_fields_foreach(e) {
+	/* First free it in all open designs */
+	rnd_conf_fields_foreach_master(e)
+		if (strncmp(e->key, prefix, len) == 0)
+			rnd_conf_multi_remove_field_per_dsg(e->key);
+
+	/* Then check if we still have anything in master (single-design apps may
+	   have free'd it in master already) */
+	rnd_conf_fields_foreach_master(e) {
 		if (strncmp(e->key, prefix, len) == 0) {
 			rnd_conf_free_native(e->value);
-			htsp_delentry(rnd_conf_fields, e);
+			htsp_delentry(rnd_conf_fields_master, e);
 		}
 	}
 }
 
 void rnd_conf_unreg_field(rnd_conf_native_t *field)
 {
-	htsp_pop(rnd_conf_fields, field->hash_path);
-	rnd_conf_free_native(field);
+	const char *hash_path = field->hash_path;
+
+	rnd_conf_multi_remove_field_per_dsg(hash_path);
+
+	/* check if we still have anything in master (single-design apps may
+	   have free'd it in master already) */
+	if (htsp_has(rnd_conf_fields_master, hash_path)) {
+		htsp_pop(rnd_conf_fields_master, hash_path);
+		rnd_conf_free_native(field);
+	}
 }
 
 
@@ -1621,7 +1696,11 @@ int rnd_conf_set_dry(rnd_conf_role_t target, const char *path_, int arr_idx, con
 			nn = lht_dom_node_alloc(LHT_TEXT, "");
 
 		if (pol == RND_POL_OVERWRITE) {
-			if (idx == -1) {
+			if (new_val == NULL) {
+				lht_tree_del(cwd); /* overwrite with NULL means: remove the whole list */
+				goto done;
+			}
+			else if (idx == -1) {
 				/* empty the list so that we insert to an empty list which is overwriting the list */
 				while(cwd->data.list.first != NULL)
 					lht_tree_del(cwd->data.list.first);
@@ -1696,7 +1775,11 @@ int rnd_conf_set_dry(rnd_conf_role_t target, const char *path_, int arr_idx, con
 	else
 		lht_tree_del(cwd);
 
+	done:;
+
 	rnd_conf_lht_dirty[target]++;
+	if (rnd_conf_edits_lock == 0)
+		rnd_conf_lht_edits[target]++;
 
 	free(path);
 	return 0;
@@ -1964,7 +2047,7 @@ int rnd_conf_replace_subtree(rnd_conf_role_t dst_role, const char *dst_path, rnd
 }
 
 
-int rnd_conf_save_file(rnd_hidlib_t *hidlib, const char *project_fn, const char *design_fn, rnd_conf_role_t role, const char *fn)
+int rnd_conf_save_file(rnd_design_t *hidlib, const char *project_fn, const char *design_fn, rnd_conf_role_t role, const char *fn)
 {
 	int fail = 1;
 	lht_node_t *r = rnd_conf_lht_get_first(role, 0);
@@ -2007,7 +2090,7 @@ int rnd_conf_save_file(rnd_hidlib_t *hidlib, const char *project_fn, const char
 				path = rnd_build_fn(hidlib, fn);
 
 			if (path == NULL) {
-				rnd_message(RND_MSG_ERROR, "Error: failed to calculate the project file name (board file name or allocation error)\n");
+				rnd_message(RND_MSG_ERROR, "Error: failed to calculate the project file name (design file name or allocation error)\n");
 				fclose(f);
 				return -1;
 			}
@@ -2044,7 +2127,7 @@ int rnd_conf_save_file(rnd_hidlib_t *hidlib, const char *project_fn, const char
 	return fail;
 }
 
-int rnd_conf_export_to_file(rnd_hidlib_t *hidlib, const char *fn, rnd_conf_role_t role, const char *conf_path)
+int rnd_conf_export_to_file(rnd_design_t *hidlib, const char *fn, rnd_conf_role_t role, const char *conf_path)
 {
 	lht_node_t *at = rnd_conf_lht_get_at(role, conf_path, 0);
 	lht_err_t r;
@@ -2154,16 +2237,22 @@ rnd_conf_role_t rnd_conf_lookup_role(const lht_node_t *nd)
 	return RND_CFR_invalid;
 }
 
+void rnd_conf_free(rnd_conf_role_t target)
+{
+	if (rnd_conf_main_root[target] != NULL) {
+		lht_dom_uninit(rnd_conf_main_root[target]);
+		rnd_conf_main_root[target] = NULL;
+	}
+}
+
 void rnd_conf_reset(rnd_conf_role_t target, const char *source_fn)
 {
 	lht_node_t *n, *cfr;
 
-	if (rnd_conf_main_root[target] != NULL)
-		lht_dom_uninit(rnd_conf_main_root[target]);
+	rnd_conf_free(target);
 
 	rnd_conf_main_root[target] = lht_dom_init();
 
-
 	if (source_fn != NULL)
 		lht_dom_loc_newfile(rnd_conf_main_root[target], source_fn);
 
@@ -2284,7 +2373,7 @@ int rnd_conf_print_native(rnd_conf_pfn pfn, void *ctx, const char * prefix, int
 
 
 /****************/
-static int conf_anyload_subtree(const rnd_anyload_t *al, rnd_hidlib_t *hl, lht_node_t *root)
+static int conf_anyload_subtree(const rnd_anyload_t *al, rnd_design_t *hl, lht_node_t *root)
 {
 	lht_doc_t *doc;
 	int res;
@@ -2334,7 +2423,7 @@ void rnd_conf_uninit(void)
 
 	rnd_anyload_unreg_by_cookie(conf_cookie_al);
 
-	rnd_conf_fields_foreach(e) {
+	rnd_conf_fields_foreach_master(e) {
 		if (strncmp(e->key, "plugins/", 8) == 0)
 			fprintf(stderr, "%s conf ERROR: conf node '%s' is not unregistered\n", rnd_app.package, e->key);
 	}
@@ -2348,11 +2437,22 @@ void rnd_conf_uninit(void)
 			lht_dom_uninit(rnd_conf_plug_root[n]);
 	}
 
-	rnd_conf_fields_foreach(e) {
+	if (rnd_conf_fields == rnd_conf_fields_master)
+		rnd_conf_fields = NULL;
+
+	rnd_conf_fields_foreach_master(e) {
 		rnd_conf_free_native(e->value);
-		htsp_delentry(rnd_conf_fields, e);
+		htsp_delentry(rnd_conf_fields_master, e);
+	}
+	htsp_free(rnd_conf_fields_master);
+
+	if (rnd_conf_fields != NULL) {
+		rnd_conf_fields_foreach(e) {
+			rnd_conf_free_native(e->value);
+			htsp_delentry(rnd_conf_fields, e);
+		}
+		htsp_free(rnd_conf_fields);
 	}
-	htsp_free(rnd_conf_fields);
 
 	vmst_uninit(&merge_subtree);
 	rnd_conf_files_uninit();
@@ -2402,4 +2502,4 @@ void rnd_conf_ro(const char *path)
 }
 
 #include "conf_regfile.c"
-#include "conf_multi_temp.c"
+#include "conf_multi.c"
