@@ -20,8 +20,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -34,10 +34,11 @@ struct rnd_box_s {        /* a bounding box */
 	rnd_coord_t X2, Y2;     /* and lower right corner */
 };
 
-typedef struct rnd_hidlib_s rnd_hidlib_t;
+typedef struct rnd_project_s rnd_project_t;
+typedef struct rnd_design_s rnd_design_t;
 
 
-/* typedef ... rnd_coord_t; pcb base unit, typedef'd in config.h */
+/* typedef ... rnd_coord_t; librnd base coord unit, typedef'd in config.h */
 typedef double rnd_angle_t; /* degrees */
 typedef struct rnd_unit_s rnd_unit_t;
 
@@ -80,6 +81,7 @@ typedef long int rnd_layergrp_id_t;
 
 typedef struct rnd_conf_ignore_s rnd_conf_ignore_t;
 typedef struct rnd_conf_native_s rnd_conf_native_t;
+typedef struct rnd_conf_state_s rnd_conf_state_t;
 
 #include <librnd/core/rnd_bool.h>
 
