@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -28,4 +28,4 @@
 #include <librnd/core/hid_cfg.h>
 /* Run an action node. The node is either a list of text nodes or a text node;
    returns non-zero on error, the first action that fails in a chain breaks the chain */
-int rnd_hid_cfg_action(rnd_hidlib_t *hl, const lht_node_t *node);
+int rnd_hid_cfg_action(rnd_design_t *hl, const lht_node_t *node);
