@@ -1,5 +1,5 @@
 #include "xm_tree_table_widget.h"
-#include <librnd/core/hid_dad_tree.h>
+#include <librnd/hid/hid_dad_tree.h>
 
 typedef struct {
 	lesstif_attr_dlg_t *ctx;
@@ -365,7 +365,8 @@ static void ltf_tt_xevent_cb(const tt_table_event_data_t *data)
 		case ett_mouse_btn_drag:
 			break;
 		case ett_mouse_btn_down:
-			XtSetKeyboardFocus(lt->ctx->dialog, lt->w);
+			if (lt->ctx->dialog != NULL) /* docked subdialog */
+				XtSetKeyboardFocus(lt->ctx->dialog, lt->w);
 			e = ltf_tt_lookup_row(data, data->current_row);
 			if (e == NULL)
 				return;
