@@ -17,14 +17,15 @@
 #include <librnd/core/error.h>
 #include <librnd/core/rnd_printf.h>
 #include <librnd/core/plugins.h>
-#include <librnd/core/hid.h>
-#include <librnd/core/hid_init.h>
-#include <librnd/core/hid_attrib.h>
+#include <librnd/hid/hid.h>
+#include <librnd/hid/hid_init.h>
+#include <librnd/hid/hid_attrib.h>
 
 #include "media.h"
 #include "draw_ps.h"
 
 static void use_gc(rnd_ps_t *pctx, rnd_hid_gc_t gc);
+extern int rnd_ps_faded;
 
 typedef struct rnd_hid_gc_s {
 	rnd_core_gc_t core_gc;
@@ -33,7 +34,6 @@ typedef struct rnd_hid_gc_s {
 	rnd_coord_t width;
 	unsigned char r, g, b;
 	int erase;
-	int faded;
 } rnd_hid_gc_s;
 
 void rnd_ps_start_file(rnd_ps_t *pctx, const char *swver)
@@ -49,7 +49,7 @@ void rnd_ps_start_file(rnd_ps_t *pctx, const char *swver)
 
 	/* %%Title DCS provides text title for the document that is useful
 	   for printing banner pages. */
-	fprintf(f, "%%%%Title: %s\n", rnd_hid_export_fn(pctx->hidlib->filename));
+	fprintf(f, "%%%%Title: %s\n", rnd_hid_export_fn(pctx->hidlib->loadname));
 
 	/* %%CreationDate DCS indicates the date and time the document was
 	   created. Neither the date nor time need be in any standard
@@ -136,7 +136,7 @@ void rnd_ps_end_file(rnd_ps_t *pctx)
 	fprintf(f, "%%%%EOF\n");
 }
 
-void rnd_ps_init(rnd_ps_t *pctx, rnd_hidlib_t *hidlib, FILE *f, int media_idx, int fillpage, double scale_factor)
+void rnd_ps_init(rnd_ps_t *pctx, rnd_design_t *hidlib, FILE *f, int media_idx, int fillpage, double scale_factor)
 {
 	memset(pctx, 0, sizeof(rnd_ps_t));
 
@@ -153,13 +153,13 @@ void rnd_ps_init(rnd_ps_t *pctx, rnd_hidlib_t *hidlib, FILE *f, int media_idx, i
 	pctx->scale_factor = scale_factor;
 	if (pctx->fillpage) {
 		double zx, zy;
-		if (hidlib->size_x > hidlib->size_y) {
-			zx = (double)pctx->ps_height / (double)hidlib->size_x;
-			zy = (double)pctx->ps_width / (double)hidlib->size_y;
+		if (rnd_dwg_get_size_x(hidlib) > rnd_dwg_get_size_y(hidlib)) {
+			zx = (double)pctx->ps_height / (double)rnd_dwg_get_size_x(hidlib);
+			zy = (double)pctx->ps_width / (double)rnd_dwg_get_size_y(hidlib);
 		}
 		else {
-			zx = (double)pctx->ps_height / (double)hidlib->size_y;
-			zy = (double)pctx->ps_width / (double)hidlib->size_x;
+			zx = (double)pctx->ps_height / (double)rnd_dwg_get_size_y(hidlib);
+			zy = (double)pctx->ps_width / (double)rnd_dwg_get_size_x(hidlib);
 		}
 		pctx->scale_factor *= MIN(zx, zy);
 	}
@@ -175,6 +175,7 @@ void rnd_ps_begin_toc(rnd_ps_t *pctx)
 {
 	/* %%Page DSC requires both a label and an ordinal */
 	fprintf(pctx->outf, "%%%%Page: TableOfContents 1\n");
+	fprintf(pctx->outf, "matrix defaultmatrix setmatrix\n"); /* resets transformation so that catting multiple page files into one is meaningful */
 	fprintf(pctx->outf, "/Times-Roman findfont 24 scalefont setfont\n");
 	fprintf(pctx->outf, "/rightshow { /s exch def s stringwidth pop -1 mul 0 rmoveto s show } def\n");
 	fprintf(pctx->outf, "/y 72 9 mul def /toc { 100 y moveto show /y y 24 sub def } bind def\n");
@@ -212,7 +213,7 @@ static void corner(FILE * fh, rnd_coord_t x, rnd_coord_t y, int dx, int dy)
 	   Postscript's "device thin" line - i.e. zero width means one
 	   device pixel.  The code remains in case you want to make them
 	   thicker - it needs to offset everything so that the *edge* of the
-	   thick line lines up with the edge of the board, not the *center*
+	   thick line lines up with the edge of the design, not the *center*
 	   of the thick line. */
 
 	rnd_fprintf(fh, "gsave %mi setlinewidth %mi %mi translate %d %d scale\n", thick * 2, x, y, dx, dy);
@@ -228,7 +229,7 @@ int rnd_ps_printed_toc(rnd_ps_t *pctx, int group, const char *name)
 		if ((group < 0) || (group != pctx->lastgroup)) {
 			if (pctx->pagecount == 1) {
 				time_t currenttime = time(NULL);
-				fprintf(pctx->outf, "30 30 moveto (%s) show\n", rnd_hid_export_fn(pctx->hidlib->filename));
+				fprintf(pctx->outf, "30 30 moveto (%s) show\n", rnd_hid_export_fn(pctx->hidlib->loadname));
 
 				fprintf(pctx->outf, "(%d.) tocp\n", pctx->pagecount);
 				fprintf(pctx->outf, "(Table of Contents \\(This Page\\)) toc\n");
@@ -283,6 +284,7 @@ int rnd_ps_new_file(rnd_ps_t *pctx, FILE *new_f, const char *fn)
 double rnd_ps_page_frame(rnd_ps_t *pctx, int mirror_this, const char *layer_fn, int noscale)
 {
 	double boffset;
+	rnd_coord_t midx, midy;
 
 	/* %%Page DSC comment marks the beginning of the PostScript
 	   language instructions that describe a particular
@@ -294,13 +296,14 @@ double rnd_ps_page_frame(rnd_ps_t *pctx, int mirror_this, const char *layer_fn,
 		gds_t tmp;
 		gds_init(&tmp);
 		fprintf(pctx->outf, "%%%%Page: %s %d\n", layer_fn, pctx->pagecount);
+		fprintf(pctx->outf, "matrix defaultmatrix setmatrix\n"); /* resets transformation so that catting multiple page files into one is meaningful */
 		gds_uninit(&tmp);
 	}
 
 	fprintf(pctx->outf, "/Helvetica findfont 10 scalefont setfont\n");
 	if (pctx->legend) {
 		gds_t tmp;
-		fprintf(pctx->outf, "30 30 moveto (%s) show\n", rnd_hid_export_fn(pctx->hidlib->filename));
+		fprintf(pctx->outf, "30 30 moveto (%s) show\n", rnd_hid_export_fn(pctx->hidlib->loadname));
 
 		gds_init(&tmp);
 		if (pctx->hidlib->name)
@@ -322,7 +325,8 @@ double rnd_ps_page_frame(rnd_ps_t *pctx, int mirror_this, const char *layer_fn,
 	rnd_fprintf(pctx->outf, "72 72 scale %mi %mi translate\n", pctx->media_width / 2, pctx->media_height / 2);
 
 	boffset = pctx->media_height / 2;
-	if (pctx->hidlib->size_x > pctx->hidlib->size_y) {
+	TODO("make this optional: this is auto-portrait");
+	if (rnd_dwg_get_size_x(pctx->hidlib) > rnd_dwg_get_size_y(pctx->hidlib)) {
 		fprintf(pctx->outf, "90 rotate\n");
 		boffset = pctx->media_width / 2;
 		if ((pctx->calibration_y != 0) && (pctx->calibration_x != 0))
@@ -335,8 +339,10 @@ double rnd_ps_page_frame(rnd_ps_t *pctx, int mirror_this, const char *layer_fn,
 		fprintf(pctx->outf, "1 -1 scale\n");
 
 
+	midx = (pctx->hidlib->dwg.X1 + pctx->hidlib->dwg.X2) / 2;
+	midy = (pctx->hidlib->dwg.Y1 + pctx->hidlib->dwg.Y2) / 2;
 	fprintf(pctx->outf, "%g dup neg scale\n", noscale ? 1.0 : pctx->scale_factor);
-	rnd_fprintf(pctx->outf, "%mi %mi translate\n", -pctx->hidlib->size_x / 2, -pctx->hidlib->size_y / 2);
+	rnd_fprintf(pctx->outf, "%mi %mi translate\n", -midx, -midy);
 
 	return boffset;
 }
@@ -354,19 +360,20 @@ void rnd_ps_page_background(rnd_ps_t *pctx, int has_outline, int page_is_route,
 
 	if (!has_outline || pctx->invert) {
 		if (page_is_route) {
-			rnd_fprintf(pctx->outf,
-								"0 setgray %mi setlinewidth 0 0 moveto 0 "
-								"%mi lineto %mi %mi lineto %mi 0 lineto closepath %s\n",
-								min_wid,
-								pctx->hidlib->size_y, pctx->hidlib->size_x, pctx->hidlib->size_y, pctx->hidlib->size_x, pctx->invert ? "fill" : "stroke");
+			rnd_fprintf(pctx->outf, "0 setgray %mi setlinewidth", min_wid);
+			rnd_fprintf(pctx->outf, " %mi %mi moveto ", pctx->hidlib->dwg.X1, pctx->hidlib->dwg.Y1);
+			rnd_fprintf(pctx->outf, " %mi %mi moveto ", pctx->hidlib->dwg.X1, pctx->hidlib->dwg.Y2);
+			rnd_fprintf(pctx->outf, " %mi %mi moveto ", pctx->hidlib->dwg.X2, pctx->hidlib->dwg.Y2);
+			rnd_fprintf(pctx->outf, " %mi %mi moveto ", pctx->hidlib->dwg.X2, pctx->hidlib->dwg.Y1);
+			rnd_fprintf(pctx->outf, " closepath %s\n", pctx->invert ? "fill" : "stroke");
 		}
 	}
 
 	if (pctx->align_marks) {
-		corner(pctx->outf, 0, 0, -1, -1);
-		corner(pctx->outf, pctx->hidlib->size_x, 0, 1, -1);
-		corner(pctx->outf, pctx->hidlib->size_x, pctx->hidlib->size_y, 1, 1);
-		corner(pctx->outf, 0, pctx->hidlib->size_y, -1, 1);
+		corner(pctx->outf, pctx->hidlib->dwg.X1, pctx->hidlib->dwg.Y1, -1, -1);
+		corner(pctx->outf, pctx->hidlib->dwg.X2, pctx->hidlib->dwg.Y1, 1, -1);
+		corner(pctx->outf, pctx->hidlib->dwg.X2, pctx->hidlib->dwg.Y2, 1, 1);
+		corner(pctx->outf, pctx->hidlib->dwg.X1, pctx->hidlib->dwg.Y2, -1, 1);
 	}
 
 	pctx->linewidth = -1;
@@ -440,11 +447,6 @@ void rnd_ps_set_draw_xor(rnd_hid_gc_t gc, int xor_)
 	;
 }
 
-void rnd_ps_set_draw_faded(rnd_hid_gc_t gc, int faded)
-{
-	gc->faded = faded;
-}
-
 static void use_gc(rnd_ps_t *pctx, rnd_hid_gc_t gc)
 {
 	pctx->drawn_objs++;
@@ -476,7 +478,7 @@ static void use_gc(rnd_ps_t *pctx, rnd_hid_gc_t gc)
 		fprintf(pctx->outf, "%d setlinecap %d setlinejoin\n", c, c);
 		pctx->lastcap = gc->cap;
 	}
-#define CBLEND(gc) (((gc->r)<<24)|((gc->g)<<16)|((gc->b)<<8)|(gc->faded))
+#define CBLEND(gc) (((gc->r)<<24)|((gc->g)<<16)|((gc->b)<<8)|(rnd_ps_faded))
 	if (pctx->lastcolor != CBLEND(gc)) {
 		if (pctx->is_drill || pctx->is_mask) {
 			fprintf(pctx->outf, "%d gray\n", (gc->erase || pctx->is_mask) ? 0 : 1);
@@ -487,7 +489,7 @@ static void use_gc(rnd_ps_t *pctx, rnd_hid_gc_t gc)
 			r = gc->r;
 			g = gc->g;
 			b = gc->b;
-			if (gc->faded) {
+			if (rnd_ps_faded) {
 				r = (1 - pctx->fade_ratio) *255 + pctx->fade_ratio * r;
 				g = (1 - pctx->fade_ratio) *255 + pctx->fade_ratio * g;
 				b = (1 - pctx->fade_ratio) *255 + pctx->fade_ratio * b;
@@ -641,7 +643,7 @@ void rnd_ps_fill_rect(rnd_ps_t *pctx, rnd_hid_gc_t gc, rnd_coord_t x1, rnd_coord
 }
 
 
-void rnd_ps_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, int action)
+void rnd_ps_set_crosshair(rnd_hid_t *hid, rnd_coord_t x, rnd_coord_t y, rnd_set_crosshair_t action)
 {
 }
 
