@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -29,6 +29,7 @@
 
 #include "compat.h"
 #include <librnd/core/hid_cfg_action.h>
+#include <librnd/hid/hid_menu.h>
 #include <librnd/plugins/lib_gtk_common/rnd_gtk.h>
 #include <librnd/plugins/lib_gtk_common/in_keyboard.h>
 #include <librnd/plugins/lib_hid_common/menu_helper.h>
@@ -58,7 +59,7 @@ static void maybe_set_chkbtn(GtkWidget *chk, int v)
 	gtk_check_button_set_active(GTK_CHECK_BUTTON(chk), v);
 }
 
-static void menu_item_update_chkbox(rnd_hidlib_t *hidlib, lht_node_t *mnd, GtkWidget *real_row)
+static void menu_item_update_chkbox(rnd_design_t *hidlib, lht_node_t *mnd, GtkWidget *real_row)
 {
 	int v;
 	GtkWidget *w, *lab, *chk, *hbox = gtk_widget_get_first_child(real_row);
@@ -90,7 +91,7 @@ static void menu_item_update_chkbox(rnd_hidlib_t *hidlib, lht_node_t *mnd, GtkWi
 		maybe_set_chkbtn(chk, !!v);
 }
 
-static void menu_update_toggle_state(rnd_hidlib_t *hidlib, open_menu_t *om)
+static void menu_update_toggle_state(rnd_design_t *hidlib, open_menu_t *om)
 {
 	int n;
 	GtkWidget *real_row;
@@ -104,7 +105,7 @@ static void menu_update_toggle_state(rnd_hidlib_t *hidlib, open_menu_t *om)
 	}
 }
 
-void rnd_gtk_main_menu_update_toggle_state(rnd_hidlib_t *hidlib, GtkWidget *menubar)
+void rnd_gtk_main_menu_update_toggle_state(rnd_design_t *hidlib, GtkWidget *menubar)
 {
 	open_menu_t *om;
 	for(om = gdl_first(&open_menu); om != NULL; om = om->link.next)
@@ -707,7 +708,7 @@ static void gtkci_menu_install_hotkeys(rnd_gtk_menu_ctx_t *menu, const lht_node_
 	}
 }
 
-GtkWidget *rnd_gtk_load_menus(rnd_gtk_menu_ctx_t *menu, rnd_hidlib_t *hidlib)
+GtkWidget *rnd_gtk_load_menus(rnd_gtk_menu_ctx_t *menu, rnd_design_t *hidlib)
 {
 	const lht_node_t *mr;
 	GtkWidget *menu_bar = NULL;
