@@ -20,8 +20,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 #ifndef RND_GTK4_COMPAT_H
@@ -434,7 +434,7 @@ static inline GtkWidget *wrap_bind_click(GtkWidget *w, struct gtkc_event_xyz_s *
 }
 
 int gtkc_clipboard_set_text(GtkWidget *widget, const char *text);
-int gtkc_clipboard_get_text(GtkWidget *wdg, void **data, size_t *len);
+char *gtkc_clipboard_get_text(GtkWidget *wdg);
 
 #ifdef GDK_WINDOWING_X11
 #include <gdk/x11/gdkx.h>
