@@ -19,8 +19,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  *  Contact:
- *    Project page: http://repo.hu/projects/pcb-rnd
- *    lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+ *    Project page: http://www.repo.hu/projects/librnd
+ *    lead developer: http://www.repo.hu/projects/librnd/contact.html
  *    mailing list: pcb-rnd (at) list.repo.hu (send "subscribe")
  */
 
@@ -54,7 +54,7 @@ static void error_draw_points_pre(GLfloat *pts) { }
 static void error_draw_points(int npts) { }
 static void error_draw_points_post(void) { }
 static void error_draw_lines6(GLfloat *pts, int npts, float red, float green, float blue, float alpha) { }
-static void error_expose_init(int w, int h, const rnd_color_t *bg_c) { }
+static void error_expose_init(int x0, int y0, int w, int h, const rnd_color_t *bg_c) { }
 static void error_set_view(double tx, double ty, double zx, double zy, double zz) { }
 static int error_xor_start(void) { return -1; }
 static void error_xor_end(void) { }
