@@ -28,8 +28,10 @@ typedef pa_big_coord_t pa_big_angle_t;
 
 int pa_angle_equ(pa_big_angle_t a, pa_big_angle_t b);
 int pa_angle_lt(pa_big_angle_t a, pa_big_angle_t b);
+int pa_angle_gt(pa_big_angle_t a, pa_big_angle_t b);
 int pa_angle_gte(pa_big_angle_t a, pa_big_angle_t b);
 int pa_angle_lte(pa_big_angle_t a, pa_big_angle_t b);
+void pa_angle_sub(pa_big_angle_t res, pa_big_angle_t a, pa_big_angle_t b); /* res = a-b */
 int pa_angle_valid(pa_big_angle_t a);
 
 
@@ -62,6 +64,7 @@ rnd_vnode_t *pa_big_node_add_single(rnd_vnode_t *dst, pa_big_vector_t ptv);
 
 /* Fill ion cd->angle assuming big coord */
 void pa_big_calc_angle(pa_conn_desc_t *cd, rnd_vnode_t *pt, char poly, char side);
+void pa_big_calc_angle_nn(pa_big_angle_t *dst, rnd_vnode_t *nfrom, rnd_vnode_t *nto);
 
 
 void pa_big_load(pa_big_coord_t dst, rnd_coord_t src);
@@ -89,8 +92,10 @@ double pa_big2_double(pa_big2_coord_t crd);
 double pa_big_vnxd(rnd_vnode_t *vn);
 double pa_big_vnyd(rnd_vnode_t *vn);
 
-/* Contour postprocessing; see pa_bool_postproc() for explanation */
-void pa_big_bool_postproc(rnd_polyarea_t **pa, int from_selfisc);
+/* Contour postprocessing; see pa_bool_postproc() for explanation. Set
+   papa_touch_risk to 1 if there were overlapping islands within A or B
+   during a non-SUB operation */
+void pa_big_bool_postproc(rnd_polyarea_t **pa, int from_selfisc, int papa_touch_risk);
 
 
 void rnd_pa_big_load_cvc(pa_big_vector_t *dst, rnd_vnode_t *src);
@@ -105,6 +110,12 @@ int pa_big2_coord_cmp(pa_big2_coord_t a, pa_big2_coord_t b);
 
 int pa_small_big_xy_eq(rnd_coord_t smallx, rnd_coord_t smally, pa_big_coord_t bigx, pa_big_coord_t bigy);
 
+
+/* Return 1 if a and vnode are very close; when they are close and copy is 1,
+   modify a to be the same as vnode's coord */
+int pa_big_vmandist_small(pa_big_vector_t *a, rnd_vnode_t *vnode, int copy);
+
+
 /* Round to nearest integer */
 void pa_big_round(pa_big_coord_t big);
 
