@@ -204,7 +204,7 @@ static rnd_r_dir_t pa_cin_crossing_small(pa_cin_ctx_t *p, pa_seg_t *s)
 #define bigcmp(a, b) pa_big_coord_cmp(a, b)
 
 /* same as above, but with big coords */
-static rnd_r_dir_t pa_cin_crossing_big(pa_cin_ctx_t *p, pa_seg_t *s)
+RND_INLINE rnd_r_dir_t pa_cin_crossing_big(pa_cin_ctx_t *p, pa_seg_t *s)
 {
 	pa_big_vector_t sv, sv_next;
 
@@ -333,6 +333,29 @@ int pa_pline_is_point_inside(const rnd_pline_t *pl, rnd_vector_t pt)
 	return ctx.f;
 }
 
+int pa_pline_is_point_on_seg(pa_seg_t *s, rnd_vector_t pt)
+{
+	pa_cin_ctx_t ctx;
+#define ON_EDGE_COOKIE 1234
+
+	/* run a horizontal ray from the point to x->infinity and count (in ctx.f)
+	   how it crosses poly edges with different winding */
+	ctx.f = 0;
+	ctx.point_on_edge_is_in = ON_EDGE_COOKIE;
+	ctx.p[0] = pt[0];
+	ctx.p[1] = pt[1];
+	ctx.p_is_big = ctx.p_has_big = 0;
+	ctx.compatibility = 0;
+
+#ifdef PA_BIGCOORD_ISC
+	pa_cin_crossing_big(&ctx, s);
+#else
+	pa_cin_crossing_small(&ctx, s)
+#endif
+
+	return ctx.f == ON_EDGE_COOKIE;
+}
+
 
 /* Returns whether pt is within the bbox of pl */
 rnd_bool pa_is_vnode_in_pline_box(const rnd_pline_t *pl, const rnd_vnode_t *nd)
@@ -406,7 +429,7 @@ int pa_pline_is_vnode_inside(const rnd_pline_t *pl, const rnd_vnode_t *nd, int p
    [O'Rourke]: Computational Geometry in C (2nd Ed.)
                Joseph O'Rourke, Cambridge University Press 1998,
                ISBN 0-521-64010-5 Pbk, ISBN 0-521-64976-5 Hbk */
-RND_INLINE void pa_pline_interior_pt(rnd_pline_t *poly, rnd_vector_t v)
+RND_INLINE int pa_pline_interior_pt(rnd_pline_t *poly, rnd_vector_t v)
 {
 	rnd_vnode_t *pt1, *pt2, *pt3, *n, *min_n = NULL;
 	double dist, min_dist = 0.0, dir = (poly->flg.orient == RND_PLF_DIR) ? 1.0 : -1.0;
@@ -449,6 +472,8 @@ RND_INLINE void pa_pline_interior_pt(rnd_pline_t *poly, rnd_vector_t v)
 		v[0] = (pt2->point[0] + min_n->point[0]) / 2;
 		v[1] = (pt2->point[1] + min_n->point[1]) / 2;
 	}
+
+	return point_in_triangle(pt1->point, pt2->point, pt3->point, v) ? 0 : -1;
 }
 
 rnd_bool pa_pline_inside_pline(rnd_pline_t *outer, rnd_pline_t *inner)
@@ -473,10 +498,33 @@ rnd_bool pa_pline_inside_pline(rnd_pline_t *outer, rnd_pline_t *inner)
 		return 0;
 #endif
 
+	/* pa_pline_interior_pt() tends to fail for tiny polygons because of the
+	   constraint that it needs to pick an integer point: if the poly is a
+	   very small triangle, the "inner" point typically falls slightly outside.
+	   Do a slower but more reliable check corner by corner. Use majority vote
+	   because one point may be outside while majority of the poly is inside.
+	   Related test case: gixedo, gixedq* */
+	if (inner->area < 6) {
+		broke:;
+		{
+			rnd_vnode_t *n = inner->head;
+			int p_in = 0, p_out = 0;
+			do {
+				int r = pa_pline_is_point_inside(outer, n->point);
+				if (r)
+					p_in++;
+				else
+					p_out++;
+			} while((n = n->next) != inner->head);
+			return (p_in >= p_out);
+		}
+	}
+
 	/* ...but that may still be only a shared point while the rest of inner
 	   is simply all outside. Since they are not intersecting if a random
 	   internal point of inner is inside outer, the whole inner is inside outer */
-	pa_pline_interior_pt(inner, ipt);
+	if (pa_pline_interior_pt(inner, ipt) != 0)
+		goto broke; /* evade corner cases where pa_pline_interior_pt() breaks */
 	return pa_pline_is_point_inside(outer, ipt);
 }
 
