@@ -150,6 +150,9 @@ static int pa_coll_jump(rnd_vnode_t **cur, pa_direction_t *cdir, pa_jump_rule_t
 		pa_direction_t newdir = *cdir;
 		rnd_vnode_t *e = d->parent;
 
+		if (d->ignore) /* skip over stubs */
+			continue;
+
 		if (d->side == 'P')
 			e = e->prev;
 
@@ -239,10 +242,11 @@ RND_INLINE int pa_coll_gather(rnd_vnode_t *start, rnd_pline_t **result, pa_jump_
 /*	orig_dir = pa_pline_update_big2small(*result);
 	rnd_trace("Bad orig=%d ", orig_dir);*/
 	pa_pline_update(*result, 1);
+#ifdef PA_BIGCOORD_ISC
 	pa_big_area_incremental(big_area, first_nd, prev_nd);
+
 	orig_dir = pa_big_area2ori(big_area);
 
-#ifdef PA_BIGCOORD_ISC
 	/* triangle flip special case; test case: fixedd, fixedi. The implicit rounding when
 	   nd is copied to newnd and cvc high resolution isc is dropped may cause
 	   self intersections in polygons that have more than 3 corners (this is
@@ -255,6 +259,11 @@ RND_INLINE int pa_coll_gather(rnd_vnode_t *start, rnd_pline_t **result, pa_jump_
 		if ((*result)->flg.orient != orig_dir)
 			pa_pline_invert(*result);
 	}
+	else {
+		/* Determine pline orient from the high resolution input instead of
+		   pa_pline_update()'s low resolution area computation. Test case: gixed2 */
+		(*result)->flg.orient = orig_dir;
+	}
 #endif
 
 	return pa_err_ok;
@@ -514,7 +523,7 @@ RND_INLINE rnd_bool pa_contour_in_box(rnd_pline_t *pl, rnd_box_t box)
 RND_INLINE void pa_polyarea_update_primary_del_inside(jmp_buf *e, rnd_box_t box, rnd_polyarea_t **islands, rnd_pline_t **holes, rnd_polyarea_t *bpa)
 {
 	rnd_polyarea_t *pa, *panext;
-	rnd_pline_t *pl;
+	rnd_pline_t *pl, *ph;
 	int finished = 0;
 
 	for(pa = *islands; !finished && (*islands != NULL); pa = panext) {
@@ -527,6 +536,9 @@ RND_INLINE void pa_polyarea_update_primary_del_inside(jmp_buf *e, rnd_box_t box,
 			/* Delete this contour, move all children to the pending holes list */
 
 			pl = pa->contours;
+			for(ph = pl->next; ph != NULL; ph = ph->next)
+				ph->flg.orphaned = 1; /* test case: gixedy */
+
 			remove_contour(pa, NULL, pl, rnd_false);
 			pa_pline_free(&pl);
 			/* rtree deleted in pa_polyarea_free_all() */
@@ -544,7 +556,7 @@ RND_INLINE void pa_polyarea_update_primary_del_inside(jmp_buf *e, rnd_box_t box,
 			}
 
 			pa_polyarea_unlink(islands, pa);
-			pa_polyarea_free_all(&pa);
+ 			pa_polyarea_free_all(&pa);
 			continue;
 		}
 
@@ -666,7 +678,7 @@ RND_INLINE void pa_polyarea_update_primary(jmp_buf *e, rnd_polyarea_t **islands,
 	assert(0);
 }
 
-RND_INLINE void pa_polyarea_collect_separated(jmp_buf *e, rnd_pline_t *A, rnd_polyarea_t **contours, rnd_pline_t **holes, int op, rnd_bool maybe)
+RND_INLINE void pa_polyarea_collect_separated(jmp_buf *e, rnd_pline_t *A, rnd_polyarea_t **contours, rnd_pline_t **holes, int op, rnd_bool maybe, rnd_polyarea_t *B)
 {
 	rnd_pline_t **pl, **next;
 
@@ -675,6 +687,20 @@ RND_INLINE void pa_polyarea_collect_separated(jmp_buf *e, rnd_pline_t *A, rnd_po
 		if (pa_collect_contour(e, pl, contours, holes, op, NULL, NULL, NULL))
 			next = pl; /* if a contour is removed, don't advance "next" twice */
 	}
+
+	/* Island got removed because being fully under another, bit leaving behind
+	   a hole; this hole did not get labelled but if it intersects b, it will
+	   still change the final output. Test case: bdale1 and bdale2 */
+	{
+		rnd_pline_t *pl, **next;
+		for(pl = *holes; pl != NULL; pl = next) {
+			next = pl->next;
+			if ((pl->flg.orphaned) && (pl->flg.llabel == PA_PLL_ISECTED) && (pl->head->flg.plabel == PA_PTL_UNKNWN)) {
+				pa_label_pline_vs_polyarea(pl, B, 0);
+				pa_collect_contour(e, &pl, contours, holes, op, NULL, NULL, NULL);
+			}
+		}
+	}
 }
 
 static void pa_polyarea_collect(jmp_buf *e, rnd_polyarea_t *A, rnd_polyarea_t **contours, rnd_pline_t **holes, int op, rnd_bool maybe)
