@@ -186,10 +186,79 @@ static pa_conn_desc_t *pa_polyarea_list_intersected(jmp_buf *e, pa_conn_desc_t *
 	return conn_list;
 }
 
+/* Corner case: if a sloped line of polygon B crosses a triangular double-slope
+   of polygon A very close to the corner of A, it may yield a short stub of
+   A. Example case: gixedk at point 'o' at 331.9999;471:
+
+               329;454
+                 |B
+                 |
+              A  |
+    319;475  ----o------x 332;471
+                /|    A
+              /  |B
+            /A   |
+      331;477    333.03;476.83
+
+   In collect we are coming from 331;477 and should leave toward 319;475.
+   This doesnt happen if the stub between o and x exists because then pa_label
+   will think the upper B segment is outside.
+
+   When a stub detected, mark both cvcs to be ignored and whenever the code
+   needs to iterate around the cvcs of a point, skip over ignored cvcs.
+*/
+
+static void pa_conn_list_remove_stubs(pa_conn_desc_t **head)
+{
+	pa_conn_desc_t *n = *head, *n2, *next;
+
+	if ((n == NULL) || (n == n->next))
+		return;
+
+	do {
+		next = n->next;
+
+		/* a stub has two edges of the same poly in the same direction... */
+		if ((n->poly == n->next->poly) && pa_angle_equ(n, n->next)) {
+			rnd_vnode_t *remote_curr, *remote_next;
+
+			n2 = n->next;
+
+			remote_curr = n->side == 'N' ? n->parent->next : n->parent->prev;
+			remote_next = n2->side == 'N' ? n2->parent->next : n2->parent->prev;
+
+			/* ...reaching the same remote node */
+			if (remote_curr == remote_next)
+				n->ignore = n2->ignore = 1; /* ignore this stub */
+		}
+	} while((n = next) != *head);
+}
+
+#define PA_ISECTED(x) ((x)->contours->flg.llabel & PA_PLL_ISECTED)
+static int pa_polyarea_assess_papa_touch_risk(rnd_polyarea_t *start, const char poly_id)
+{
+	rnd_polyarea_t *a = start, *b;
+	do {
+/*		rnd_trace("yfixed8 %c: %d\n", poly_id, PA_ISECTED(a));*/
+		if (PA_ISECTED(a)) {
+			b = a->f;
+			do {
+				if (pa_polyarea_box_overlap(a, b)) {
+/*					rnd_trace(" box overlap!\n");*/
+					return 1;
+				}
+			}
+			while ((b = b->f) != start);
+		}
+	} while ((a = a->f) != start);
+	return 0;
+}
+
+
 /* Compute intersections between all islands of pa_a and pa_b. If all_iscs is
    true, collect and label all intersections, else stop after the first
    (usefule if only the fact of the intersection is interesting) */
-static void pa_polyarea_intersect(jmp_buf *e, rnd_polyarea_t *pa_a, rnd_polyarea_t *pa_b, int all_iscs)
+static void pa_polyarea_intersect(jmp_buf *e, rnd_polyarea_t *pa_a, rnd_polyarea_t *pa_b, int op, int all_iscs, int *papa_touch_risk)
 {
 	rnd_polyarea_t *a = pa_a, *b = pa_b;
 	pa_conn_desc_t *conn_list = NULL;
@@ -215,9 +284,20 @@ static void pa_polyarea_intersect(jmp_buf *e, rnd_polyarea_t *pa_a, rnd_polyarea
 		pa_debug_print_cvc(conn_list);
 	} while (all_iscs && (b = b->f) != pa_b);
 
+	/* Corner case: fixedy8. If more than one island of the same polyare
+	   participates in a non-sub operation, if those islands have overlapping
+	   edges (valid) after the operation a single island of the resulting poly
+	   will have overlapping edges (invalid). Mark this risk if:
+	     - pa_a or pa_b has multiple isnalds intersected
+	     - and any two of those islands have overlapping bboxes
+	*/
+	if ((papa_touch_risk != NULL) && (op != RND_PBO_SUB))
+		*papa_touch_risk = (pa_polyarea_assess_papa_touch_risk(pa_a, 'A') || pa_polyarea_assess_papa_touch_risk(pa_b, 'B'));
+
 	/* add all intersections of all of pa_a's islands to the conn list */
 	do {
 		conn_list = pa_polyarea_list_intersected(e, conn_list, a, 'A');
+		pa_conn_list_remove_stubs(&conn_list);
 		pa_debug_print_cvc(conn_list);
 	} while (all_iscs && ((a = a->f) != pa_a));
 
@@ -263,6 +343,27 @@ int pa_cvc_crossing_at_node(rnd_vnode_t *nd)
 		);*/
 
 	if ((cn->parent != nd) && (cp->parent != nd)) {
+		if (cn->parent != cp->parent) {
+			/* Special case: test case gixedo2 around 152;626:
+			          144;616
+			          +,
+			            \
+			   me        '\
+			   92;650      '-----+ 155;627
+			   +-----------+-----+ 154;628
+			              /'\
+			            /'   '\
+			          +'       '+
+			      141;644      155;632
+			
+			   The check is coming from 92;65 and figures 155;627 and 155;632->*
+			   are the next and prev incoming nodes into this CVC. This looks like
+			   a crossing because they are on different * point than 'me'. However
+			   they are not on the same different path, so that path is not crossing
+			   'me'. */
+			return 0;
+		}
+		
 		/* if either neighbour is our own node, that means we could proceed that
 		   way from this crossing, which means it's a >< topology. If both
 		   neighbours are some other node's cvc, we are in a X crossing, going
