@@ -148,6 +148,8 @@ static void rnd_poly_copy_edge_tree_(rnd_rtree_t *dst, rnd_rtree_t *parent, cons
 
 void rnd_poly_copy_edge_tree(rnd_pline_t *dst, const rnd_pline_t *src)
 {
+	if (src->tree == NULL)
+		return;
 	/* copy the tree recursively */
 	dst->tree = rnd_r_create_tree();
 	rnd_poly_copy_edge_tree_(dst->tree, NULL, src->tree, dst);
@@ -305,7 +307,7 @@ static rnd_r_dir_t seg_in_seg_cb(const rnd_box_t *b, void *cl)
 	pa_seg_seg_t *ctx = (pa_seg_seg_t *)cl;
 	pa_seg_t *s = (pa_seg_t *)b;
 	pa_big_vector_t isc1, isc2;
-	int num_isc;
+	int num_isc, adjacent = 0;
 
 	/* When new nodes are added at the end of a pass due to an intersection
 	   the segments may be altered. If either segment we're looking at has
@@ -313,10 +315,21 @@ static rnd_r_dir_t seg_in_seg_cb(const rnd_box_t *b, void *cl)
 	if (s->intersected || ctx->s->intersected)
 		return RND_R_DIR_NOT_FOUND;
 
+	/* ignore overlap if the two edges are the same */
+	if (s->v == ctx->v)
+		return RND_R_DIR_NOT_FOUND;
+
+	if ((s->v == ctx->v->next) || (s->v->next == ctx->v))
+		adjacent = 1;
+
 	num_isc = pa_isc_edge_edge_(s->v, s->v->next, ctx->v, ctx->v->next, &isc1, &isc2);
 	if (!num_isc)
 		return RND_R_DIR_NOT_FOUND;
 
+	/* ignore adjacent edge single endpoint overlap, but don't ignore longer overlaps */
+	if ((num_isc == 1) && adjacent)
+		return RND_R_DIR_NOT_FOUND;
+
  /* if checking touches the first intersection decides the result */
 	if (ctx->touch != NULL)
 		longjmp(*ctx->touch, PA_ISC_TOUCHES);
@@ -331,6 +344,11 @@ static rnd_r_dir_t seg_in_seg_cb(const rnd_box_t *b, void *cl)
 		pa_big_vector_t *my_s = (num_isc == 1 ? &isc1 : &isc2);
 		rnd_vnode_t *new_node;
 
+		/* If intersection is very close to any of the endpoints, rather modify
+		   the intersection point to match that endpoint. This avoids a lot of nasty
+		   corner cases with shortest possible edges. Test case: gixedr* */
+		pa_big_vmandist_small(my_s, ctx->v, 1) || pa_big_vmandist_small(my_s, ctx->v->next, 1) || pa_big_vmandist_small(my_s, s->v, 1) || pa_big_vmandist_small(my_s, s->v->next, 1);
+
 		/* add new node on "i" */
 		new_node = pa_ensure_point_and_prealloc_cvc(ctx->v, *my_s);
 		pa_debug_print_isc2(num_isc, "ctx", my_s, new_node);
