@@ -37,6 +37,31 @@ typedef struct {
 	rnd_pline_t *neg_head, *neg_tail;
 } pa_posneg_t;
 
+/* Reset a polyline back to pre-mapping state; enable tree rebuild if
+   nodes have been added or removed. */
+static void pl_remove_cvcs(rnd_pline_t *pl, rnd_bool rebuild_tree)
+{
+	rnd_vnode_t *n = pl->head;
+
+	do {
+		if (n->cvclst_prev != NULL) {
+			free(n->cvclst_prev);
+			free(n->cvclst_next);
+			n->cvclst_prev = n->cvclst_next = NULL;
+			/* no need to unlink, all cvcs are free'd here */
+		}
+
+		n->shared = NULL;
+		n->flg.mark = n->flg.start = 0;
+	} while((n = n->next) != pl->head);
+
+	if (rebuild_tree) {
+		rnd_r_free_tree_data(pl->tree, free);
+		rnd_r_destroy_tree(&pl->tree);
+		pl->tree = rnd_poly_make_edge_tree(pl);
+	}
+}
+
 static void posneg_append_pline(pa_posneg_t *posneg, int polarity, rnd_pline_t *pl)
 {
 	assert(pl != NULL);
@@ -127,13 +152,13 @@ RND_INLINE int on_same_seg_mark(rnd_vnode_t *va, rnd_vnode_t *vb)
 	if (on_same_pt(va, vb) && on_same_pt(va->next, vb->next)) {
 		va->shared = vb;
 		vb->shared = va;
-		rnd_trace("     shared+={%p} {%p}", va, vb);
+		DEBUG_SELFISC("     shared+={%p} {%p}", va, vb);
 		return 1;
 	}
 	if (on_same_pt(va->next, vb) && on_same_pt(va, vb->next)) {
 		va->shared = vb;
 		vb->shared = va;
-		rnd_trace("     shared+={%p} {%p}", va, vb);
+		DEBUG_SELFISC("     shared+={%p} {%p}", va, vb);
 		return 1;
 	}
 	return 0;
@@ -156,12 +181,12 @@ static rnd_r_dir_t pa_selfisc_map_cross_cb(const rnd_box_t *b, void *cl)
 	if ((s->v == ctx->v) || (s->v == ctx->v->next) || (s->v == ctx->v->prev))
 		return RND_R_DIR_NOT_FOUND;
 
-rnd_trace("  ISC vs %.2f;%.2f  %.2f;%.2f:\n", NODE_CRDS(s->v), NODE_CRDS(s->v->next));
+DEBUG_SELFISC("  ISC vs %.2f;%.2f  %.2f;%.2f:\n", NODE_CRDS(s->v), NODE_CRDS(s->v->next));
 
 	if (on_same_seg_mark(ctx->v, s->v)) {
 		/* avoid infinite loop of re-discovering the same shared segment, plus
 		   optimization for trivial shared lines */
-		rnd_trace("   shared seg marked (quick)\n");
+		DEBUG_SELFISC("   shared seg marked (quick)\n");
 		ctx->num_isc++;
 		return RND_R_DIR_NOT_FOUND;
 	}
@@ -185,11 +210,11 @@ rnd_trace("  ISC vs %.2f;%.2f  %.2f;%.2f:\n", NODE_CRDS(s->v), NODE_CRDS(s->v->n
 		return RND_R_DIR_NOT_FOUND;
 
 
-rnd_trace("   [%d] %.2f;%.2f", num_isc, pa_big_double(isc1.x), pa_big_double(isc1.y));
+DEBUG_SELFISC("   [%d] %.2f;%.2f", num_isc, pa_big_double(isc1.x), pa_big_double(isc1.y));
 if (num_isc > 1)
-	rnd_trace("  and   %.2f;%.2f\n", pa_big_double(isc2.x), pa_big_double(isc2.y));
+	DEBUG_SELFISC("  and   %.2f;%.2f\n", pa_big_double(isc2.x), pa_big_double(isc2.y));
 else
-	rnd_trace("\n");
+	DEBUG_SELFISC("\n");
 
 	/* Having two intersections means line-line overlap: class 5 */
 	if (num_isc == 2) {
@@ -232,7 +257,7 @@ else
 		shb->shared = sha;
 		sha->shared = shb;
 
-		rnd_trace("   shared seg marked (alloced=%d)\n", alloced);
+		DEBUG_SELFISC("   shared seg marked (alloced=%d)\n", alloced);
 		ctx->num_isc++;
 		if (alloced)
 			return RND_R_DIR_CANCEL; /* modified the tree, need to restart the search at this node */
@@ -289,7 +314,7 @@ static rnd_r_dir_t pa_selfisc_install_cvc_cb(const rnd_box_t *b, void *cl)
 		ctx->v->cvclst_next = pa_prealloc_conn_desc(pt);
 		list = pa_add_conn_desc_at(ctx->v, 's', NULL);
 
-		rnd_trace("  include initial:  %ld;%ld - %ld;%ld\n",
+		DEBUG_SELFISC("  include initial:  %ld;%ld - %ld;%ld\n",
 			(long)ctx->v->point[0], (long)ctx->v->point[1],
 			(long)ctx->v->next->point[0], (long)ctx->v->next->point[1]);
 	}
@@ -298,7 +323,7 @@ static rnd_r_dir_t pa_selfisc_install_cvc_cb(const rnd_box_t *b, void *cl)
 		memcpy(&pt, &ctx->v->cvclst_prev->isc, sizeof(pt));
 	}
 
-	rnd_trace("  include offender: %ld;%ld - %ld;%ld\n",
+	DEBUG_SELFISC("  include offender: %ld;%ld - %ld;%ld\n",
 		(long)offender->point[0], (long)offender->point[1],
 		(long)offender->next->point[0], (long)offender->next->point[1]);
 
@@ -320,14 +345,37 @@ RND_INLINE int pa_eff_dir_forward(char dir1, char dir2)
 	return eff_dir;
 }
 
+
+/* Return startnd if it's not marked or if it is marked but has no cvc links.
+   If it's marked and has cvc links return another, unmarked node form the
+   cvc list for this point. If no unmarked forund, return startnd.
+   Used to revisit a node that multiple entries, see test case gixedt. */
+RND_INLINE rnd_vnode_t *get_cvc_unmarked(rnd_vnode_t *startnd)
+{
+	pa_conn_desc_t *c, *start;
+
+	if (!startnd->flg.mark)
+		return startnd;
+
+	start = startnd->cvclst_next;
+	if (start == NULL)
+		return startnd;
+
+	for(c = start->next; c != start; c = c->next)
+		if (!c->parent->flg.mark)
+			return c->parent;
+
+	return startnd;
+}
+
 /* Step from n to the next node according to dir, walking the outline (trying
    to achieve largest area) */
-RND_INLINE rnd_vnode_t *pa_selfisc_next_o(rnd_vnode_t *n, char *dir)
+RND_INLINE rnd_vnode_t *pa_selfisc_next_o(rnd_vnode_t *n, char *dir, rnd_vnode_t *prev_node)
 {
 	pa_conn_desc_t *c, *start;
 	rnd_vnode_t *onto, *shtest;
 
-	rnd_trace(" next: ");
+	DEBUG_SELFISC(" next: ");
 	if (n->cvclst_prev == NULL) {
 		if (*dir == 'N') {
 			onto = n->next;
@@ -337,13 +385,13 @@ RND_INLINE rnd_vnode_t *pa_selfisc_next_o(rnd_vnode_t *n, char *dir)
 			onto = n->prev;
 			onto->flg.mark = 1;
 		}
-		rnd_trace("straight to %.2f %.2f\n", NODE_CRDS(onto));
+		DEBUG_SELFISC("straight to %.2f %.2f\n", NODE_CRDS(onto));
 
 		return onto;
 	}
 
 	start = c = n->cvclst_prev->next;
-	rnd_trace("CVC (c->side=%c *dir=%c)\n", c->side, *dir);
+	DEBUG_SELFISC("CVC (c->side=%c *dir=%c)\n", c->side, *dir);
 	do {
 		int eff_dir = pa_eff_dir_forward(c->side, *dir);
 
@@ -353,11 +401,20 @@ RND_INLINE rnd_vnode_t *pa_selfisc_next_o(rnd_vnode_t *n, char *dir)
 		}
 		else shtest = onto = c->parent->prev;
 
-		rnd_trace("  %.2f %.2f '%c' eff_dir=%d sh?={%p}", NODE_CRDS(onto), c->side, eff_dir, c->parent);
-		if (shtest->shared) {
-			rnd_trace(" refuse (shared)\n");
+		DEBUG_SELFISC("  %.2f %.2f '%c' eff_dir=%d sh?={%p}", NODE_CRDS(onto), c->side, eff_dir, c->parent);
+
+		if ((onto == prev_node) || ((prev_node != NULL) && on_same_pt(onto, prev_node))) {
+			/* test case: gixed1b at 730;408 not going back to 738;399 */
+			DEBUG_SELFISC(" refuse (prev_node, coming from)\n");
 			continue;
 		}
+
+		if (shtest->shared) {
+			DEBUG_SELFISC(" refuse (shared)\n");
+			continue;
+		}
+		if (onto->flg.mark)
+			onto = get_cvc_unmarked(onto);
 		if (!onto->flg.mark) {
 			*dir = eff_dir ? 'N' : 'P';
 			if (eff_dir)
@@ -365,10 +422,10 @@ RND_INLINE rnd_vnode_t *pa_selfisc_next_o(rnd_vnode_t *n, char *dir)
 			else
 				onto->flg.mark = 1;
 			
-			rnd_trace(" accept, dir '%c' (onto) {%p}!\n", *dir, onto);
+			DEBUG_SELFISC(" accept, dir '%c' (onto) {%p}!\n", *dir, onto);
 			return onto;
 		}
-		rnd_trace(" refuse (marked) {%p}\n", onto);
+		DEBUG_SELFISC(" refuse (marked) {%p}\n", onto);
 	} while((c = c->prev) != start);
 
 	/* corner case: bowtie-kind of self-isc with the original 'start' node
@@ -385,7 +442,7 @@ RND_INLINE rnd_vnode_t *pa_selfisc_next_o(rnd_vnode_t *n, char *dir)
 		else onto = c->parent->prev;
 
 		if (onto->flg.start) {
-			rnd_trace(" accept, dir '%c' (start %.2f;%.2f)!\n", *dir, NODE_CRDS(onto));
+			DEBUG_SELFISC(" accept, dir '%c' (start %.2f;%.2f)!\n", *dir, NODE_CRDS(onto));
 			if (eff_dir)
 				c->parent->flg.mark = 1;
 			else
@@ -398,10 +455,119 @@ RND_INLINE rnd_vnode_t *pa_selfisc_next_o(rnd_vnode_t *n, char *dir)
 	abort();
 }
 
+
+/* Stub detection: next node has a cvc and the previous and next nodes are at
+	   the same place, so there are two overlapping lines from and to 'start'.
+	   In this case start from the next point and when we reach prev simply
+	   stop so that this stub is excluded.
+	
+	   This needs to be done in a loop because there may be a chain of stubs
+	   (and we are starting from the far end), see gixedm5:
+	
+	   *  556;421 <- starting point
+	   |
+	   *  556;422
+	   |
+	   *  556;423 <- node pair; new start is the right point, stop_at is the left
+	   |'\
+	   |  '\
+	   *----'* 558;425
+	   557;
+	   425
+	
+	   Other stubs elsewhere on the loop are safely discarded by
+	   pa_selfisc_next_o(), this affects the starting point only.
+*/
+RND_INLINE int is_node_in_stub(rnd_vnode_t **start, rnd_vnode_t **sprev, rnd_vnode_t **stop_at)
+{
+	if (((*start)->next->cvclst_next != NULL) && pa_vnode_equ(*sprev, (*start)->next)) {
+		*stop_at = *sprev;
+		(*start)->flg.mark = 1; /* don't start at this node */
+		*start = (*start)->next;
+		*sprev = (*sprev)->prev;
+		return 1;
+	}
+
+	/* Special case: gixedm6: if there's a longer chain the last-minus-1st node
+	   won't look like a stub because the last node of the stub has empty cvc */
+	if (((*start)->cvclst_next != NULL) && ((*start)->next->cvclst_next == NULL) && pa_vnode_equ(*start, (*start)->next->next)) {
+		/* jump onto the next node, assuming it's the end of the stub */
+		(*start)->flg.mark = 1; /* don't start at this node */
+		*start = (*start)->next;
+		*sprev = (*start)->prev;
+		*stop_at = *sprev;
+		return 1;
+	}
+
+	return 0;
+}
+
+/* Stubs have to be removed before outline collection because a stub can
+   overlap a legit edge that needs to be kept. This is done in this
+   preprocessing step. The function returns start or NULL. When NULL is
+   returned, the polyline has been modified and needs to be re-mapped.
+   Related test case: gixedp; legit edge is:
+     (39;292) 48;289 49;289 50;289 (52;289)
+   offending overlapping stub is:
+     (52;288) 50;289 49;289 48;289 49;289 50;289 (52;287)
+
+   A simpler test case is si_class6
+
+   This also fixed fixedm4.
+*/
+static rnd_vnode_t *stub_remover(pa_selfisc_t *ctx, rnd_vnode_t *start)
+{
+	rnd_vnode_t *n, *loop_start, *sprev, *stop_at, *next, *removed1, *removed2;
+
+/*	DEBUG_SELFISC("STUB removal\n");*/
+
+	/* don't start from a stub */
+	loop_start = start;
+	sprev = start->prev;
+	while(is_node_in_stub(&loop_start, &sprev, &stop_at)) ;
+
+	n = loop_start;
+	do {
+		bypass_restart:;
+		next = n->next;
+		if ((n->next->cvclst_next != NULL) && (n->prev->cvclst_next != NULL) && pa_vnode_equ(n->prev, n->next)) {
+			pa_conn_desc_t *c1, *c2;
+
+			/* n is the endpoint of a stub */
+/*			DEBUG_SELFISC("STUB  found endpoint at: %ld;%ld prev=%ld;%ld next=%ld;%ld\n", n->point[0], n->point[1], n->prev->point[0], n->prev->point[1], n->next->point[0], n->next->point[1]);*/
+
+			if (n == start)
+				start = n->next;
+
+/*DEBUG_SELFISC("    stub removal1: %ld;%ld -> %ld;%ld\n", n->point[0], n->point[1], n->next->point[0], n->next->point[1]);*/
+			removed1 = n;
+			rnd_poly_vertex_exclude(ctx->pl, n);
+
+			/* now we have two points at the new stub endpoint: next and next->prev;
+			   Remove one of them. */
+			n = next;
+			next = n->prev;
+/*DEBUG_SELFISC("    stub removal2: %ld;%ld -> %ld;%ld\n", n->point[0], n->point[1], n->next->point[0], n->next->point[1]);*/
+			removed2 = n;
+			rnd_poly_vertex_exclude(ctx->pl, n);
+
+			start = NULL;
+
+			if ((loop_start == removed1) || (loop_start == removed2)) {
+				loop_start = next;
+				n = next;
+				goto bypass_restart;
+			}
+		}
+	} while((n = next) != loop_start);
+
+	return start;
+}
+
 /* Collect the outline, largest area possible; remember islands cut off */
 RND_INLINE void pa_selfisc_collect_outline(pa_posneg_t *posneg, rnd_pline_t *src, rnd_vnode_t *start)
 {
-	rnd_vnode_t *n, *last, *newn;
+	rnd_vnode_t *n, *last, *newn, *prev2, *prev_node = NULL;
 	rnd_pline_t *dst;
 	char dir = 'N';
 
@@ -410,15 +576,18 @@ RND_INLINE void pa_selfisc_collect_outline(pa_posneg_t *posneg, rnd_pline_t *src
 	start->flg.start = 1;
 	dst = pa_pline_new(start->point);
 
-	rnd_trace("selfi collect outline from %.2f %.2f\n", NODE_CRDS(start));
+	DEBUG_SELFISC("selfi collect outline from %.2f %.2f\n", NODE_CRDS(start));
 
 	/* append dst to the list of plines */
 	posneg_append_pline(posneg, +1, dst);
 
 	/* collect a closed loop */
 	last = dst->head;
-	for(n = pa_selfisc_next_o(start, &dir); (n != start) && (n != NULL); n = pa_selfisc_next_o(n, &dir)) {
-		rnd_trace(" at out %.2f %.2f {%p}", NODE_CRDS(n), n);
+
+	n = pa_selfisc_next_o(start, &dir, NULL);
+	prev2 = n;
+	for(; (n != start) && (n != NULL); n = pa_selfisc_next_o(n, &dir, prev_node)) {
+		DEBUG_SELFISC(" at out %.2f %.2f {%p}", NODE_CRDS(n), n);
 		/* Can't assert for this: in the bowtie case the same crossing point has two roles
 			assert(!n->flg.mark); (should face marked nodes only as outgoing edges of intersections)
 		*/
@@ -427,23 +596,26 @@ RND_INLINE void pa_selfisc_collect_outline(pa_posneg_t *posneg, rnd_pline_t *src
 		newn->point[1] = n->point[1];
 		rnd_poly_vertex_include(last, newn);
 		last = newn;
+
+		prev_node = prev2;
+		prev2 = n;
 	}
 	start->flg.start = 0;
 }
 
 /* Step from n to the next node according to dir, walking a hole island (trying to
    get minimal area portions) */
-RND_INLINE rnd_vnode_t *pa_selfisc_next_i(rnd_vnode_t *n, char *dir, rnd_vnode_t **first, rnd_vnode_t *real_start)
+RND_INLINE rnd_vnode_t *pa_selfisc_next_i(rnd_vnode_t *n, char *dir, rnd_vnode_t **first, rnd_vnode_t *real_start, rnd_vnode_t *prev_node)
 {
 	pa_conn_desc_t *c, *start;
 	rnd_vnode_t *onto, *shtest;
 
-	rnd_trace("    next (first:%p dir='%c'): ", first, *dir);
+	DEBUG_SELFISC("    next (first:%p dir='%c'): ", first, *dir);
 	if (n->cvclst_prev == NULL) {
 		if (*dir == 'N') {
 			onto = n->prev;
 			onto->flg.mark = 1;
-rnd_trace("[mark %.2f;%.2f] ", NODE_CRDS(onto));
+DEBUG_SELFISC("[mark %.2f;%.2f] ", NODE_CRDS(onto));
 			if (first) {
 				*first = onto;
 				onto->flg.start = 1;
@@ -452,21 +624,24 @@ rnd_trace("[mark %.2f;%.2f] ", NODE_CRDS(onto));
 		else {
 			onto = n->next;
 			n->flg.mark = 1;
-rnd_trace("[mark %.2f;%.2f] ", NODE_CRDS(n));
+DEBUG_SELFISC("[mark %.2f;%.2f] ", NODE_CRDS(n));
 			if (first) {
 				*first = n;
 				n->flg.start = 1;
 			}
 		}
-		rnd_trace("straight to %.2f %.2f\n", NODE_CRDS(onto));
+		DEBUG_SELFISC("straight to %.2f %.2f\n", NODE_CRDS(onto));
 		return onto;
 	}
 
-	rnd_trace("CVC\n");
+	DEBUG_SELFISC("CVC\n");
 
 	/* quick return: if we can get back to the starting point from this CVC, do
-	   that; other loops will be picked up separately. Test case: gixedj */
-	if (real_start != NULL) {
+	   that; other loops will be picked up separately. Test case: gixedj. But
+	   don't do it if it's the previous node we are coming from because
+	   that would only create a stub. Test case: gixeds:
+	   start = prev_node = 365;519  n = 364;516 (cvc)  onto = 365;519  -> rather proceed elsewhere */
+	if ((real_start != NULL) && (real_start != prev_node)) {
 		start = c = n->cvclst_prev->next;
 		do {
 			int eff_dir = pa_eff_dir_forward(c->side, *dir);
@@ -493,17 +668,25 @@ rnd_trace("[mark %.2f;%.2f] ", NODE_CRDS(n));
 			shtest = c->parent;
 		}
 
-		rnd_trace("     %.2f %.2f '%c' m%d s%d %d onto={%p} parent={%p}", NODE_CRDS(onto), c->side, onto->flg.mark, onto->flg.start, c->parent->flg.start, onto, c->parent);
+		DEBUG_SELFISC("     %.2f %.2f '%c' m%d s%d %d onto={%p} parent={%p}", NODE_CRDS(onto), c->side, onto->flg.mark, onto->flg.start, c->parent->flg.start, onto, c->parent);
 		if (c->parent->flg.start) {
 			if (first) continue; /* tried to start onto an already mapped path; try the next */
 			return NULL; /* arrived back to the starting point - finish normally */
 		}
 
-		if (shtest->shared) {
-			rnd_trace(" refuse (shared)\n");
+		if (onto == prev_node) { /* test case: gixed1 */
+			DEBUG_SELFISC(" refuse (prev_node, coming from)\n");
 			continue;
 		}
 
+		if (shtest->shared) {
+			DEBUG_SELFISC(" refuse (shared)\n");
+			continue;
+		}
+
+		if (onto->flg.mark)
+			onto = get_cvc_unmarked(onto);
+
 		if (!onto->flg.mark || onto->flg.start) { /* also accept flg.start here: greedy algorithm to find shortest loops */
 			if (!onto->flg.start) /* don't remember new direction going out from the endpoint; test case: gixedbo2 */
 				*dir = eff_dir ? 'N' : 'P';
@@ -513,7 +696,7 @@ rnd_trace("[mark %.2f;%.2f] ", NODE_CRDS(n));
 					*first = onto;
 					onto->flg.start = 1;
 				}
-				rnd_trace(" mark %.2f %.2f M1a s%d {%p}", NODE_CRDS(onto), first, onto);
+				DEBUG_SELFISC(" mark %.2f %.2f M1a s%d {%p}", NODE_CRDS(onto), first, onto);
 			}
 			else {
 				c->parent->flg.mark = 1;
@@ -521,13 +704,13 @@ rnd_trace("[mark %.2f;%.2f] ", NODE_CRDS(n));
 					*first = c->parent;
 					c->parent->flg.start = 1;
 				}
-				rnd_trace(" mark %.2f %.2f M1b s%d {%p}", NODE_CRDS(c->parent), first, c->parent);
+				DEBUG_SELFISC(" mark %.2f %.2f M1b s%d {%p}", NODE_CRDS(c->parent), first, c->parent);
 			}
 
-			rnd_trace("    accept, dir '%c' (onto)!\n", *dir);
+			DEBUG_SELFISC("    accept, dir '%c' (onto)!\n", *dir);
 			return onto;
 		}
-		rnd_trace("    refuse (marked)\n");
+		DEBUG_SELFISC("    refuse (marked)\n");
 	} while((c = c->prev) != start);
 
 	if (!first) {
@@ -546,23 +729,27 @@ RND_INLINE void pa_selfisc_collect_island(pa_posneg_t *posneg, rnd_vnode_t *star
 {
 	int accept_pol = 0, has_selfisc = 0;
 	char dir = 'N';
-	rnd_vnode_t *n, *newn, *last, *started = NULL;
+	rnd_vnode_t *n, *newn, *last, *started = NULL, *prev_node = NULL, *prev2;
 	rnd_pline_t *dst;
 
 	dst = pa_pline_new(start->point);
 	last = dst->head;
 
-	rnd_trace("  island {:\n");
-	rnd_trace("   IS1 %.2f %.2f\n", NODE_CRDS(start));
-	for(n = pa_selfisc_next_i(start, &dir, &started, NULL); (n != start) && (n != NULL); n = pa_selfisc_next_i(n, &dir, 0, started)) {
-		rnd_trace("   IS2 %.2f %.2f\n", NODE_CRDS(n));
+	DEBUG_SELFISC("  island {:\n");
+	DEBUG_SELFISC("   IS1 %.2f %.2f\n", NODE_CRDS(start));
+	n = pa_selfisc_next_i(start, &dir, &started, NULL, NULL);
+	prev2 = n;
+	for(; (n != start) && (n != NULL); n = pa_selfisc_next_i(n, &dir, 0, started, prev_node)) {
+		DEBUG_SELFISC("   IS2 %.2f %.2f\n", NODE_CRDS(n));
 
 		/* This is rounding n->cvc into newn->point */
 		newn = calloc(sizeof(rnd_vnode_t), 1);
 		newn->point[0] = n->point[0];
 		newn->point[1] = n->point[1];
-		rnd_trace("      appn: %ld;%ld %p\n", newn->point[0], newn->point[1], newn);
+		DEBUG_SELFISC("      appn: %ld;%ld %p\n", newn->point[0], newn->point[1], newn);
 		rnd_poly_vertex_include(last, newn);
+		prev_node = prev2;
+		prev2 = n;
 		last = newn;
 	}
 	pa_pline_update(dst, 1);
@@ -578,7 +765,7 @@ RND_INLINE void pa_selfisc_collect_island(pa_posneg_t *posneg, rnd_vnode_t *star
 		else if ((dir == 'P') && (dst->flg.orient == RND_PLF_DIR))
 			accept_pol = +1;
 	}
-	rnd_trace("\n  } (end island: len=%d dir=%c PLF=%d accept=%d)\n", dst->Count, dir, dst->flg.orient == RND_PLF_DIR, accept_pol);
+	DEBUG_SELFISC("\n  } (end island: len=%d dir=%c PLF=%d accept=%d)\n", dst->Count, dir, dst->flg.orient == RND_PLF_DIR, accept_pol);
 	if (started != NULL)
 		started->flg.start = 0;
 
@@ -604,7 +791,7 @@ RND_INLINE void pa_selfisc_collect_islands(pa_posneg_t *posneg, rnd_vnode_t *sta
 {
 	rnd_vnode_t *n;
 
-	rnd_trace("selfi collect islands from %d %d\n", start->point[0], start->point[1]);
+	DEBUG_SELFISC("selfi collect islands from %d %d\n", start->point[0], start->point[1]);
 
 	/* detect uncollected loops */
 	n = start;
@@ -614,7 +801,7 @@ RND_INLINE void pa_selfisc_collect_islands(pa_posneg_t *posneg, rnd_vnode_t *sta
 		if ((cstart == NULL) || n->flg.mark)
 			continue;
 		
-		rnd_trace(" at isl %.2f %.2f\n", NODE_CRDS(n));
+		DEBUG_SELFISC(" at isl %.2f %.2f\n", NODE_CRDS(n));
 		c = cstart;
 		do {
 			if (!c->parent->flg.mark)
@@ -628,17 +815,17 @@ RND_INLINE void pa_selfisc_collect_islands(pa_posneg_t *posneg, rnd_vnode_t *sta
 int stub_from(rnd_vnode_t *nd)
 {
 	pa_conn_desc_t *c = nd->cvclst_prev;
-/*	rnd_trace("STUB at %ld %ld:\n", nd->point[0], nd->point[1]);*/
+/*	DEBUG_SELFISC("STUB at %ld %ld:\n", nd->point[0], nd->point[1]);*/
 	do {
 #if 0
-		rnd_trace(" %f %d ", pa_big_double(c->angle), (int)c->prelim);
-		rnd_trace("P %p (%ld %ld) ", c->parent->prev, (long)c->parent->prev->point[0], (long)c->parent->prev->point[1]);
-		rnd_trace("N %p (%ld %ld)\n", c->parent->next, (long)c->parent->next->point[0], (long)c->parent->next->point[1]);
+		DEBUG_SELFISC(" %f %d ", pa_big_double(c->angle), (int)c->prelim);
+		DEBUG_SELFISC("P %p (%ld %ld) ", c->parent->prev, (long)c->parent->prev->point[0], (long)c->parent->prev->point[1]);
+		DEBUG_SELFISC("N %p (%ld %ld)\n", c->parent->next, (long)c->parent->next->point[0], (long)c->parent->next->point[1]);
 #endif
 		if (pa_big_coord_cmp(c->angle, c->prev->angle) == 0) {
 			/* test case: gixedd at 836;559 <-> 835;559 */
 TODO("arc: simply having the same angle works for lines but won't work for arcs; check if the outgoing edges fully overlap");
-			rnd_trace("(STUB detected at %ld %ld)\n", nd->point[0], nd->point[1]);
+			DEBUG_SELFISC("(STUB detected at %ld %ld)\n", nd->point[0], nd->point[1]);
 			return 1;
 		}
 	} while((c = c->next) != nd->cvclst_prev);
@@ -651,7 +838,7 @@ TODO("arc: simply having the same angle works for lines but won't work for arcs;
    Test case: fixedo at 225;215. Detect the X case and mark ctx self-isc */
 static void selfisc_detect_cvc_crossing(pa_selfisc_t *ctx, rnd_vnode_t *nd)
 {
-	rnd_trace(" CVC cross detect: %ld;%ld\n", nd->point[0], nd->point[1]);
+	DEBUG_SELFISC(" CVC cross detect: %ld;%ld\n", nd->point[0], nd->point[1]);
 
 #if 0
 	pa_conn_desc_t *c;
@@ -659,8 +846,8 @@ static void selfisc_detect_cvc_crossing(pa_selfisc_t *ctx, rnd_vnode_t *nd)
 	c = nd->cvclst_prev;
 	do {
 		rnd_vnode_t *cn = c->parent;
-		rnd_trace("  LS:   %p == %p\n", c->parent, nd);
-		rnd_trace("  ls:   %ld;%ld {%ld;%ld} %ld;%ld (%d) %c\n",
+		DEBUG_SELFISC("  LS:   %p == %p\n", c->parent, nd);
+		DEBUG_SELFISC("  ls:   %ld;%ld {%ld;%ld} %ld;%ld (%d) %c\n",
 			cn->prev->point[0], cn->prev->point[1],
 			cn->point[0], cn->point[1],
 			cn->next->point[0], cn->next->point[1],
@@ -671,14 +858,14 @@ static void selfisc_detect_cvc_crossing(pa_selfisc_t *ctx, rnd_vnode_t *nd)
 
 	if (pa_cvc_crossing_at_node(nd)) {
 		ctx->num_isc++;
-		rnd_trace("  -> X crossing\n");
+		DEBUG_SELFISC("  -> X crossing\n");
 	}
 	else if (stub_from(nd)) {
-		rnd_trace("  -> no crossing, >< topology but found stub\n");
+		DEBUG_SELFISC("  -> no crossing, >< topology but found stub\n");
 		ctx->num_isc++;
 	}
 	else
-		rnd_trace("  -> no crossing, >< topology\n");
+		DEBUG_SELFISC("  -> no crossing, >< topology\n");
 }
 
 
@@ -690,12 +877,12 @@ static rnd_vnode_t *split_selfisc_map(pa_selfisc_t *ctx)
 	/* the outline mapper needs minnode anyway, best if we figure that early
 	   so all output is deterministic */
 	n = start = pa_find_minnode(ctx->pl);
-	rnd_trace("self-isc: map_cross start\n");
+	DEBUG_SELFISC("self-isc: map_cross start\n");
 	do {
 		rnd_box_t box;
 		int rr;
 
-		rnd_trace(" map cross: %.2f;%.2f - %.2f;%.2f\n", NODE_CRDS(n), NODE_CRDS(n->next));
+		DEBUG_SELFISC(" map cross: %.2f;%.2f - %.2f;%.2f\n", NODE_CRDS(n), NODE_CRDS(n->next));
 
 		next = n->next;
 		ctx->v = n;
@@ -712,11 +899,11 @@ static rnd_vnode_t *split_selfisc_map(pa_selfisc_t *ctx)
 	   newly added points. It's (probably) the cheapest to just do a search from
 	   each point and add CVCs to any that ahs more than 2 edges coming in . */
 	n = start;
-	rnd_trace("self-isc: install cvc start\n");
+	DEBUG_SELFISC("self-isc: install cvc start\n");
 	do {
 		rnd_box_t box;
 
-		rnd_trace(" inst cvc: %.2f;%.2f\n", NODE_CRDS(n));
+		DEBUG_SELFISC(" inst cvc: %.2f;%.2f\n", NODE_CRDS(n));
 
 		n->flg.mark = 0;
 		ctx->v = n;
@@ -748,7 +935,7 @@ static rnd_vnode_t *split_selfisc_map(pa_selfisc_t *ctx)
 
 /* Build the outer contour of self-intersecting pl. Return
    whether there was a self intersection (and posneg got loaded). */
-static rnd_bool rnd_pline_split_selfisc_o(pa_posneg_t *posneg, rnd_pline_t *pl)
+static rnd_bool rnd_pline_split_selfisc_o(rnd_polyarea_t *parent, pa_posneg_t *posneg, rnd_pline_t *pl)
 {
 	rnd_vnode_t *start;
 	pa_selfisc_t ctx = {0};
@@ -759,9 +946,23 @@ static rnd_bool rnd_pline_split_selfisc_o(pa_posneg_t *posneg, rnd_pline_t *pl)
 	if (ctx.num_isc == 0)
 		return 0; /* no self intersection */
 
+	start = stub_remover(&ctx, start);
+	if (start == NULL) {
+		pl_remove_cvcs(pl, 1);
+		start = split_selfisc_map(&ctx);
+		if (ctx.num_isc == 0)
+			return 0; /* no self intersection */
+	}
+
 	/* preserve original holes as negatives */
 	if (pl->next != NULL) {
+		rnd_pline_t *h;
 		posneg_append_pline(posneg, -1, pl->next);
+
+		/* get the detached holes removed from the parent contour tree so there
+		   are no ghosts left behind; test case: gixedx */
+		for(h = pl->next; h != NULL; h = h->next)
+			rnd_r_delete_entry(parent->contour_tree, (rnd_box_t *)h);
 		pl->next = NULL;
 	}
 
@@ -798,6 +999,14 @@ static rnd_bool rnd_pline_split_selfisc_i(pa_posneg_t *posneg, rnd_polyarea_t **
 	if (ctx.num_isc == 0)
 		return 0; /* no self intersection */
 
+	start = stub_remover(&ctx, start);
+	if (start == NULL) {
+		pl_remove_cvcs(pl, 1);
+		start = split_selfisc_map(&ctx);
+		if (ctx.num_isc == 0)
+			return 0; /* no self intersection */
+	}
+
 	first_pos_null = (posneg->first_pos == NULL);
 
 	pa_selfisc_collect_islands(posneg, start);
@@ -931,8 +1140,15 @@ RND_INLINE void split_selfisc_pline_resolved(rnd_polyarea_t **pa, rnd_pline_t *p
 		/* make sure all islands are updated so they have an area */
 		for(n = 0; n < posneg->subseq_pos.used; n++) {
 			rnd_pline_t *island = posneg->subseq_pos.array[n];
-			if (island->area <= 0)
+			if (island->area <= 0) {
+				if (island->tree != NULL) { /* pa_pline_update() will recalc the tree - test case: gixedu */
+					rnd_rtree_t *r = island->tree;
+					rnd_r_free_tree_data(r, free);
+					rnd_r_destroy_tree(&r);
+					island->tree = NULL;
+				}
 				pa_pline_update(island, 0);
+			}
 		}
 
 		/* sort islands so that the largest is first; there are only
@@ -1004,7 +1220,7 @@ RND_INLINE rnd_cardinal_t split_selfisc_pline(rnd_polyarea_t **pa)
 		pa_posneg_t posneg = {0};
 		int has_selfisc;
 
-		rnd_trace("^ pa %p (f=%p in=%p)\n", *pa, (*pa)->f, pa_start);
+		DEBUG_SELFISC("^ pa %p (f=%p in=%p)\n", *pa, (*pa)->f, pa_start);
 
 		/* remember pa->f so that new positive islands we are inserting after (*pa)
 		   are not affected */
@@ -1029,7 +1245,7 @@ RND_INLINE rnd_cardinal_t split_selfisc_pline(rnd_polyarea_t **pa)
 
 		/* pline intersects itself: outline */
 		pl = (*pa)->contours;
-		has_selfisc += rnd_pline_split_selfisc_o(&posneg, pl);
+		has_selfisc += rnd_pline_split_selfisc_o(*pa, &posneg, pl);
 
 		if (has_selfisc != 0)
 			split_selfisc_pline_resolved(pa, pl, &posneg);
