@@ -88,6 +88,8 @@ struct rnd_pline_s {
 		unsigned int llabel:3; /* one of pa_pline_label_t */
 		unsigned int orient:1;
 		unsigned int risky:1;  /* need to run selfisc resolver in postproc */
+		unsigned orphaned:1;   /* parent island got deleted */
+		unsigned sliced:1;     /* used by the slicer */
 	} flg;
 };
 
@@ -112,8 +114,10 @@ void pa_pline_invert(rnd_pline_t *c); /* invert contour */
 
 rnd_vnode_t *rnd_poly_node_create(rnd_vector_t v);
 
+/* NOTE: these don't increase pline->Count */
 void rnd_poly_vertex_include(rnd_vnode_t *after, rnd_vnode_t *node);
 void rnd_poly_vertex_include_force(rnd_vnode_t *after, rnd_vnode_t *node); /* do not remove nodes even if on the same line */
+
 void rnd_poly_vertex_exclude(rnd_pline_t *parent, rnd_vnode_t *node);
 
 /* this should be an internal function but it's also used in self-isc code */
@@ -285,6 +289,13 @@ void *rnd_poly_make_edge_tree(rnd_pline_t *src);
 
 int pa_pline_is_vnode_inside(const rnd_pline_t *pl, const rnd_vnode_t *nd, int point_on_edge_is_in);
 
+/* When true, do not run assert() autocheck after operations. Useful for
+   autotest */
+extern int rnd_polybool_disable_autocheck;
+
+/* When true, assert instead of returning error code */
+extern int rnd_polybool_assert_on_error_code;
+
 /*** generic geo ***/
 int rnd_point_in_triangle(rnd_vector_t A, rnd_vector_t B, rnd_vector_t C, rnd_vector_t P);
 
