@@ -1,7 +1,5 @@
 #ifndef LIBRND_SCCONFIG_APP_TREE
-/* for compatibility */
-/* TODO:pcb1 - make this an #error in librnd4 */
-#define APP "pcb"
+#error librnd requires APP to be defined to the app name, e.g. #define APP "pcb"
 #else
 #define APP LIBRND_SCCONFIG_APP_TREE
 #endif
@@ -72,18 +70,18 @@ static int rnd_hook_custom_arg_(const char *key, const char *value, const arg_au
 		return 1;
 	}
 	if (strcmp(key, "debug") == 0) {
-		put("/local/pcb/debug", strue);
+		put("/local/rnd/debug", strue);
 		put("/local/" APP"/debug", strue);
 		pup_set_debug(strue);
 		return 1;
 	}
 	if (strcmp(key, "profile") == 0) {
-		put("/local/pcb/profile", strue);
+		put("/local/rnd/profile", strue);
 		put("/local/" APP "/profile", strue);
 		return 1;
 	}
 	if (strcmp(key, "symbols") == 0) {
-		put("/local/pcb/symbols", strue);
+		put("/local/rnd/symbols", strue);
 		put("/local/" APP "/symbols", strue);
 		return 1;
 	}
@@ -147,11 +145,11 @@ int plugin_dep1(int require, const char *plugin, const char *deps_on)
 	const char *st_plugin, *st_deps_on, *ext_deps_on;
 	int dep_chg = 0;
 
-	sprintf(buff, "/local/pcb/%s/controls", plugin);
+	sprintf(buff, "/local/module/%s/controls", plugin);
 	st_plugin = get(buff);
-	sprintf(buff, "/local/pcb/%s/external", deps_on);
+	sprintf(buff, "/local/module/%s/external", deps_on);
 	ext_deps_on = get(buff);
-	sprintf(buff, "/local/pcb/%s/controls", deps_on);
+	sprintf(buff, "/local/module/%s/controls", deps_on);
 	st_deps_on = get(buff);
 
 	/* do not change buff here, code below depends on it being set on deps_on/controls */
@@ -199,7 +197,7 @@ static void all_plugin_select(const char *state, int force)
 #undef plugin_dep
 #define plugin_def(name, desc, default_, all_) \
 	if ((all_) || force) { \
-		sprintf(buff, "/local/pcb/%s/controls", name); \
+		sprintf(buff, "/local/module/%s/controls", name); \
 		put(buff, state); \
 	}
 #define plugin_header(sect)
@@ -217,7 +215,7 @@ static int plugin_dep_set_rec1(const char *plg, const char *on, const char *pare
 	char buff[1024], *st_child;
 
 	if (strcmp(on, parent_plugin) == 0) {
-		sprintf(buff, "/local/pcb/%s/controls", plg);
+		sprintf(buff, "/local/module/%s/controls", plg);
 		st_child = get(buff);
 
 		if ((strcmp(target_state, "disable") == 0) && (strcmp(st_child, "disable") != 0)) {
@@ -280,21 +278,21 @@ int plugin_dep_ext(int require, const char *plugin, const char *deps_on)
 	if (require != 0)
 		return 0;
 
-	sprintf(buff, "/local/pcb/%s/external", deps_on);
+	sprintf(buff, "/local/module/%s/external", deps_on);
 	is_external = istrue(get(buff));
 
-	sprintf(buff, "/local/pcb/%s/externally_forced", deps_on);
+	sprintf(buff, "/local/module/%s/externally_forced", deps_on);
 	is_ext_forced = istrue(get(buff));
 	if (!is_external && !is_ext_forced)
 		return 0;
 
-	sprintf(buff, "/local/pcb/%s/controls", plugin);
+	sprintf(buff, "/local/module/%s/controls", plugin);
 	st_plugin = get(buff);
-	sprintf(buff, "/local/pcb/%s/controls", deps_on);
+	sprintf(buff, "/local/module/%s/controls", deps_on);
 	st_deps_on = get(buff);
 
 	if ((strcmp(st_deps_on, "disable") == 0) && (strcmp(st_plugin, "disable") != 0)) {
-		sprintf(buff, "/local/pcb/%s/explicit", plugin);
+		sprintf(buff, "/local/module/%s/explicit", plugin);
 		is_explicit = (get(buff) != NULL);
 
 		if (is_explicit) {
@@ -314,7 +312,7 @@ int plugin_dep_ext(int require, const char *plugin, const char *deps_on)
 	}
 
 	if ((strcmp(st_deps_on, "plugin") == 0) && (strcmp(st_plugin, "buildin") == 0)) {
-		sprintf(buff, "/local/pcb/%s/explicit", plugin);
+		sprintf(buff, "/local/module/%s/explicit", plugin);
 		is_explicit = (get(buff) != NULL);
 
 		if (is_explicit) {
@@ -336,7 +334,7 @@ int plugin_dep_ext(int require, const char *plugin, const char *deps_on)
 	/* if we had to change a plugin because of an external, mark it, so that
 	   this choice won't be overridden */
 	if (dep_chg) {
-		sprintf(buff, "/local/pcb/%s/externally_forced", plugin);
+		sprintf(buff, "/local/module/%s/externally_forced", plugin);
 		put(buff, strue);
 	}
 
@@ -381,15 +379,13 @@ void rnd_hook_postinit()
 #ifndef LIBRNDS_SCCONFIG
 	put("/local/confdir", "");
 #endif
-	put("/local/pcb/debug", sfalse);
+	put("/local/rnd/debug", sfalse);
 	put("/local/" APP "/debug", sfalse);
-	put("/local/pcb/profile", sfalse);
+	put("/local/rnd/profile", sfalse);
 	put("/local/" APP "/profile", sfalse);
-	put("/local/pcb/symbols", sfalse);
+	put("/local/rnd/symbols", sfalse);
 	put("/local/" APP "/symbols", sfalse);
-	put("/local/pcb/disable_so", sfalse);
 	put("/local/" APP "/disable_so", sfalse);
-	put("/local/pcb/want_static_librnd", sfalse);
 	put("/local/" APP "/want_static_librnd", sfalse);
 
 #undef plugin_def
@@ -412,10 +408,10 @@ static int all_plugin_check_explicit(void)
 #undef plugin_header
 #undef plugin_dep
 #define plugin_def(name, desc, default_, all_) \
-	sprintf(pwanted, "/local/pcb/%s/explicit", name); \
+	sprintf(pwanted, "/local/module/%s/explicit", name); \
 	wanted = get(pwanted); \
 	if (wanted != NULL) { \
-		sprintf(pgot, "/local/pcb/%s/controls", name); \
+		sprintf(pgot, "/local/module/%s/controls", name); \
 		got = get(pgot); \
 		if (strcmp(got, wanted) != 0) {\
 			report("ERROR: %s was requested to be %s but I had to %s it\n", name, wanted, got); \
@@ -436,7 +432,7 @@ int rnd_hook_postarg(const char *librnd_prefix, const char *app_name)
 
 #ifndef LIBRNDS_SCCONFIG
 	{
-		char *pfn = str_concat("", librnd_prefix, "/share/librnd3/plugin.state", NULL);
+		char *pfn = str_concat("", librnd_prefix, "/share/librnd4/plugin.state", NULL);
 
 		if (import(pfn) < 1) {
 			fprintf(stderr, "Failed to import librnd plugin state file from '%s'\n", pfn);
